<?php
class install_main
{
	static public function run ($settings)
	{
		$connect_info = self::_int_get_connect_info ($settings);
		if (self::_int_install_site ($settings, $connect_info) == false) {
			return (false);
		}
		if (self::_int_update_php_ini ($settings) == false) {
			return (false);
		}
		if (self::_int_update_http_conf ($settings) == false) {
			return (false);
		}
		if (self::_int_install_database ($settings, $connect_info) == false) {
			return (false);
		}
		if (self::_int_restart_httpd () == false) {
			printf ("WARNING: Failed to restart the Apache webserver. Please restart it manually.\n");
		}
		return (true);
	}

	static private function _int_install_site ($settings, $connect_info)
	{
		if (@lstat ($settings ["awc_dir_install"])) {
			if (self::_int_ask_yes_no (
				sprintf (
					"Directory \"%s\" already exists. Overwrite it?",
					$settings ["awc_dir_install"]
				),
				false
			)) {
				if (lib_file::tree_delete ($settings ["awc_dir_install"]) == false) {
					printf (
						"ERROR: Failed to delete existing directory \"%s\".\n",
						$settings ["awc_dir_install"]
					);
					return (false);
				}
			} else {
				printf (
					"ERROR: Cannot install in \"%s\" - directory already exists.\n",
					$settings ["awc_dir_install"]
				);
				return (false);
			}
		}
		if (@mkdir ($settings ["awc_dir_install"], 0755) == false) {
			printf (
				"ERROR: Failed to create the install directory \"%s\".\n",
				$settings ["awc_dir_install"]
			);
			return (false);
		}
		if (self::_int_install_users ($settings) == false) {
			return (false);
		}
		if (self::_int_install_main ($settings, $connect_info) == false) {
			return (false);
		}
		return (true);
	}

	static private function _int_install_main ($settings, $connect_info)
	{
		$dir_main = $settings ["awc_dir_install"] . "main/";
		if (lib_file::tree_copy (
			$settings ["installer_dir"] . "site/",
			$dir_main,
			array (
				"keep_ownership" => false,
				"keep_permissions" => true
			)
		) == false) {
			printf ("ERROR: Failed to copy files to \"%s\".\n", $dir_main);
			return (false);
		}
		$data =
			"<?php\n" .
			"define (\"LOG_ENABLE\", false, false);\n" .
			"define (\"DB_ENGINE\", " . sql_engine::POSTGRESQL . ", false);\n" .
			"define (\"DB_MECHANISM\", " . $connect_info ["mechanism"] . ", false);\n" .
			"define (" .
				"\"DB_CONNECT_1\", " .
				(
					($connect_info ["connect_1"] != "") ?
					"\"" . lib_misc::escape_php_string ($connect_info ["connect_1"]) . "\"" :
					"null"
				) . ", " .
				"false" .
			");\n" .
			"define (" .
				"\"DB_CONNECT_2\", " .
				(
					($connect_info ["connect_2"] != "") ?
					"\"" . lib_misc::escape_php_string ($connect_info ["connect_2"]) . "\"" :
					"null"
				) . ", " .
				"false" .
			");\n" .
			"define (" .
				"\"DB_USERNAME\", " .
				(
					($settings ["db_web_username"] != "") ?
					"\"" . lib_misc::escape_php_string ($settings ["db_web_username"]) . "\"" :
					"null"
				) . ", " .
				"false" .
			");\n" .
			"define (" .
				"\"DB_PASSWORD\", " .
				(
					($settings ["db_web_password"] != "") ?
					"\"" . lib_misc::escape_php_string ($settings ["db_web_password"]) . "\"" :
					"null"
				) . ", " .
				"false" .
			");\n" .
			"define (" .
				"\"DB_DATABASE\", " .
				"\"" . lib_misc::escape_php_string ($settings ["db_database"]) . "\", " .
				"false" .
			");\n" .
			"?>\n";
		if (@file_put_contents (
			$dir_main . "system/include/generated/config.inc",
			$data
		) !== strlen ($data)) {
			printf ("ERROR: Failed to write the AWC configuration file.\n");
			return (false);
		}
		return (true);
	}

	static private function _int_install_users ($settings)
	{
		$dir_users = $settings ["awc_dir_install"] . "users/";
		if (@mkdir ($dir_users, 0755) == false) {
			printf ("ERROR: Failed to create the users directory \"%s\".\n", $dir_users);
			return (false);
		}
		if (($apache_conf = self::_int_get_apache_conf ($settings)) === false) {
			return (false);
		}
		if (
			($apache_conf ["user"] === null) ||
			(@chown ($dir_users, $apache_conf ["user"]) == false)
		) {
			printf ("WARNING: Failed to set owner of the users directory \"%s\".\n", $dir_users);
		}
		if (
			($apache_conf ["group"] === null) ||
			(@chgrp ($dir_users, $apache_conf ["group"]) == false)
		) {
			printf ("WARNING: Failed to set group of the users directory \"%s\".\n", $dir_users);
		}
		return (true);
	}

	static private function _int_get_apache_conf ($settings)
	{
		if (($data = @file_get_contents ($settings ["apache_conf"])) === false) {
			printf (
				"WARNING: Failed to read the Apache configuration file \"%s\"",
				$settings ["apache_conf"]
			);
			return (false);
		}
		$result = array ();
		if (preg_match ('/^\\s*User\\s+(\\S+)\\s*$/m', $data, $matches)) {
			$result ["user"] = $matches [1];
		}
		if (preg_match ('/^\\s*Group\\s+(\\S+)\\s*$/m', $data, $matches)) {
			$result ["group"] = $matches [1];
		}
		return ($result);
	}

	static private function _int_update_php_ini ($settings)
	{
		if (extension_loaded ('ionCube Loader')) {
			return (true);
		}
		if (@copy (
			$settings ["php_ini"],
			$settings ["php_ini"] . ".bak"
		) == false) {
			printf (
				"ERROR: Failed to backup the PHP configuration file \"%s\".\n",
				$settings ["php_ini"]
			);
			return (false);
		}
		if (($loader = self::_int_find_loader ($settings)) === false) {
			printf ("ERROR: Failed to find a suitable IonCube loader version.\n");
			return (false);
		}
		if (($handle = fopen ($settings ["php_ini"], "a")) === false) {
			printf (
				"ERROR: Failed to open for writing the PHP configuration file \"%s\".\n",
				$settings ["php_ini"]
			);
			return (false);
		}
		$text = "\nzend_extension=" . $loader . "\n";
		$length = strlen ($text);
		if (@fwrite ($handle, $text, $length) !== $length) {
			printf (
				"ERROR: Failed to write changes to the PHP configuration file \"%s\".\n",
				$settings ["php_ini"]
			);
			@fclose ($handle);
			return (false);
		}
		if (@fclose ($handle) == false) {
			printf (
				"ERROR: Failed to close the PHP configuration file \"%s\".\n",
				$settings ["php_ini"]
			);
			return (false);
		}
		return (true);
	}

	static private function _int_find_loader ($settings)
	{
		$pathname =
			$settings ["awc_dir_install"] .
			"main/system/ioncube/ioncube_loader_lin_" .
			substr (phpversion (), 0, 3) .
			".so";
		if (file_exists ($pathname) == false) {
			return (false);
		}
		return ($pathname);
	}

	static private function _int_update_http_conf ($settings)
	{
		if (@copy (
			$settings ["apache_conf"],
			$settings ["apache_conf"] . ".bak"
		) == false) {
			printf (
				"ERROR: Failed to backup the apache configuration file \"%s\".\n",
				$settings ["apache_conf"]
			);
			return (false);
		}
		if (($handle = fopen ($settings ["apache_conf"], "a")) === false) {
			printf (
				"ERROR: Failed to open for writing the apache configuration file \"%s\".\n",
				$settings ["apache_conf"]
			);
			return (false);
		}
		$dir_htdocs = $settings ["awc_dir_install"] . "main/htdocs";
		$text =
			"\n" .
			"<VirtualHost " . $settings ["apache_ip"] . ":80>\n" .
				"\tDocumentRoot \"" . $dir_htdocs . "\"\n" .
				"\tServerName " . $settings ["apache_hostname"] . "\n" .
				"\tAlias /users/ \"" . $settings ["awc_dir_install"] . "users/" . "\"\n" .
				"\t<Directory \"" . $dir_htdocs . "\">\n" .
					"\t\tOptions FollowSymLinks\n" .
					"\t\tAllowOverride FileInfo\n" .
				"\t</Directory>\n" .
				"\t<Directory \"" . $settings ["awc_dir_install"] . "users\">\n" .
					"\t\tOptions FollowSymLinks\n" .
					"\t\tAllowOverride FileInfo\n" .
				"\t</Directory>\n" .
			"</VirtualHost>\n";
		$length = strlen ($text);
		if (@fwrite ($handle, $text, $length) !== $length) {
			printf (
				"ERROR: Failed to write changes to the apache configuration file \"%s\".\n",
				$settings ["apache_conf"]
			);
			@fclose ($handle);
			return (false);
		}
		if (@fclose ($handle) == false) {
			printf (
				"ERROR: Failed to close the apache configuration file \"%s\".\n",
				$settings ["apache_conf"]
			);
			return (false);
		}
		return (true);
	}

	static private function _int_install_database ($settings, $connect_info)
	{
		if (sql_engine_postgresql::is_supported () == false) {
			printf ("ERROR: The PostgreSQL extension is not loaded in PHP.");
			return (false);
		}
		if (self::_int_database_create ($connect_info, $settings) == false) {
			return (false);
		}
		if (self::_int_database_plpgsql ($connect_info, $settings) == false) {
			return (false);
		}
		if (self::_int_database_load ($connect_info, $settings) == false) {
			return (false);
		}
		return (true);
	}

	static private function _int_database_create ($connect_info, $settings)
	{
		$engine = sql_manager::create_engine (
			$connect_info +
			array (
				"username" => ($settings ["db_admin_username"] != "") ? $settings ["db_admin_username"] : null,
				"password" => ($settings ["db_admin_password"] != "") ? $settings ["db_admin_password"] : null,
				"database" => "template1"
			)
		);
		if (@$engine->connect () == false) {
			printf ("ERROR: Failed to connect to the PostgreSQL database (db creation).\n");
			return (false);
		}
		if (self::_int_database_create_database ($engine, $settings) == false) {
			return (false);
		}
		if (self::_int_database_create_user ($engine, $settings) == false) {
			return (false);
		}
		return (true);
	}

	static private function _int_database_create_database ($engine, $settings)
	{
		if (($rows = self::_int_query (
			$engine,
			"SELECT EXISTS (" .
				"SELECT 1 " .
				"FROM pg_catalog.pg_database " .
				"WHERE datname='" . $engine->escape ($settings ["db_database"]) . "'" .
			") AS result"
		)) === false) {
			printf ("ERROR: Failed to check existence of PostgreSQL database.\n");
			return (false);
		}
		if ($rows [0]["result"] != "f") {
			if (self::_int_ask_yes_no (
				sprintf (
					"PostgreSQL database \"%s\" already exists. Overwrite it?",
					$settings ["db_database"]
				),
				false
			)) {
				if (self::_int_query (
					$engine,
					"DROP DATABASE " . $engine->escape ($settings ["db_database"])
				) === false) {
					printf (
						"ERROR: Failed to drop existing database \"%s\".\n",
						$settings ["db_database"]
					);
					return (false);
				}
			} else {
				printf (
					"ERROR: Cannot create PostgreSQL database \"%s\" - it already exists.\n",
					$settings ["db_database"]
				);
				return (false);
			}
		}
		if (self::_int_query (
			$engine,
			"CREATE DATABASE " .
			$engine->escape ($settings ["db_database"])
		) === false) {
			printf (
				"ERROR: Failed to create PostgreSQL database \"%s\": \"%s\".\n",
				$settings ["db_database"],
				@$engine->error ()
			);
			return (false);
		}
		return (true);
	}

	static private function _int_database_create_user ($engine, $settings)
	{
		if (($rows = self::_int_query (
			$engine,
			"SELECT EXISTS (" .
				"SELECT 1 " .
				"FROM pg_catalog.pg_shadow " .
				"WHERE usename='" . $engine->escape ($settings ["db_web_username"]) . "'" .
			") AS result"
		)) === false) {
			printf ("ERROR: Failed to check for existence of PostgreSQL user.\n");
			return (false);
		}
		if ($rows [0]["result"] != "f") {
			if (self::_int_ask_yes_no (
				sprintf (
					"PostgreSQL user \"%s\" already exists. Overwrite it?",
					$settings ["db_web_username"]
				),
				false
			)) {
				if (self::_int_query (
					$engine,
					"DROP USER " . $engine->escape ($settings ["db_web_username"])
				) === false) {
					printf (
						"ERROR: Failed to drop existing user \"%s\".\n",
						$settings ["db_web_username"]
					);
					return (false);
				}
			} else {
				printf (
					"ERROR: Cannot create PostgreSQL user \"%s\" - it already exists.\n",
					$settings ["db_web_username"]
				);
				return (false);
			}
		}
		if (self::_int_query (
			$engine,
			"CREATE USER " .
			$engine->escape ($settings ["db_web_username"]) . " " .
			"UNENCRYPTED PASSWORD " .
			"'" . $engine->escape ($settings ["db_web_password"]) . "'"
		) === false) {
			printf (
				"ERROR: Failed to create PostgreSQL user \"%s\": \"%s\".\n",
				$settings ["db_database"],
				@$engine->error ()
			);
			return (false);
		}
		if (self::_int_query (
			$engine,
			"ALTER DATABASE " . $engine->escape ($settings ["db_database"]) . " " .
			"OWNER TO " . $engine->escape ($settings ["db_web_username"])
		) === false) {
			printf (
				"ERROR: Failed to grant PostgreSQL user \"%s\" access " .
				"to PostgreSQL database \"%s\": \"%s\".\n",
				$settings ["db_web_username"],
				$settings ["db_database"],
				@$engine->error ()
			);
			return (false);
		}
		return (true);
	}

	static private function _int_database_plpgsql ($connect_info, $settings)
	{
		$engine = sql_manager::create_engine (
			$connect_info +
			array (
				"username" => ($settings ["db_admin_username"] != "") ? $settings ["db_admin_username"] : null,
				"password" => ($settings ["db_admin_password"] != "") ? $settings ["db_admin_password"] : null,
				"database" => $settings ["db_database"]
			)
		);
		if (@$engine->connect () == false) {
			printf ("ERROR: Failed to connect to the PostgreSQL database (db plpgsql).\n");
			return (false);
		}
		if (($rows = self::_int_query (
			$engine,
			"SELECT EXISTS (" .
				"SELECT 1 " .
				"FROM pg_catalog.pg_language " .
				"WHERE lanname='plpgsql'" .
			") AS result"
		)) === false) {
			printf ("ERROR: Failed to check if the PostgreSQL database already has PL/pgSQL support.\n");
			return (false);
		}
		if ($rows [0]["result"] != "t") {
			if (self::_int_query (
				$engine,
				"CREATE TRUSTED LANGUAGE 'plpgsql' " .
				"HANDLER plpgsql_call_handler " .
				"VALIDATOR plpgsql_validator"
			) === false) {
				printf ("ERROR: Failed to add PL/pgSQL support to the database.\n");
				return (false);
			}
		}
		if (self::_int_query (
			$engine,
			"GRANT USAGE ON LANGUAGE plpgsql " .
			"TO " . $settings ["db_web_username"]
		) === false) {
			printf ("ERROR: Failed to grant usage on PL/pgSQL to the database user.\n");
			return (false);
		}
		return (true);
	}

	static private function _int_database_load ($connect_info, $settings)
	{
		$engine = sql_manager::create_engine (
			$connect_info +
			array (
				"username" => ($settings ["db_web_username"] != "") ? $settings ["db_web_username"] : null,
				"password" => ($settings ["db_web_password"] != "") ? $settings ["db_web_password"] : null,
				"database" => $settings ["db_database"]
			)
		);
		if (@$engine->connect () == false) {
			printf ("ERROR: Failed to connect to the PostgreSQL database (db loading).\n");
			return (false);
		}
		if (self::_int_database_load_initialize ($engine, $settings) == false) {
			return (false);
		}
		if (self::_int_database_load_update ($engine, $connect_info, $settings) == false) {
			return (false);
		}
		return (true);
	}

	static private function _int_database_load_initialize ($engine, $settings)
	{
		if (($data = @file_get_contents (
			$settings ["installer_dir"] . "site/system/docs/create.sql"
		)) === false) {
			printf ("ERROR: Failed to read the PostgreSQL initialization data.\n");
			return (false);
		}
		if (self::_int_query ($engine, $data) === false) {
			printf ("ERROR: Failed to load the PostgreSQL initialization data.\n");
			return (false);
		}
		return (true);
	}

	static private function _int_database_load_update ($engine, $connect_info, $settings)
	{
		if (self::_int_query (
			$engine,
			"INSERT INTO settings (" .
				"setting_id, " .
				"dir_temp, " .
				"pathname_zend, " .
				"pathname_ioncube, " .
				"run_dir, " .
				"run_url, " .
				"layout_dir_htdocs, " .
				"layout_dir_system, " .
				"layout_protect_system, " .
				"mysql_use, " .
				"mysql_mechanism, " .
				"mysql_connect_1, " .
				"mysql_connect_2, " .
				"mysql_admin_username, " .
				"mysql_admin_password, " .
				"mysql_client_host, " .
				"postgresql_use, " .
				"postgresql_mechanism, " .
				"postgresql_connect_1, " .
				"postgresql_connect_2, " .
				"postgresql_admin_username, " .
				"postgresql_admin_password, " .
				"sqlite_use, " .
				"sqlite_pathname" .
			") VALUES (" .
				"'1', " .
				"'" . $engine->escape ($settings ["awc_dir_temp"]) . "', " .
				(
					($settings ["awc_pathname_zend"] != "") ?
					"'" . $engine->escape ($settings ["awc_pathname_zend"]) . "'" :
					"NULL"
				) .
				", " .
				(
					($settings ["awc_pathname_ioncube"] != "") ?
					"'" . $engine->escape ($settings ["awc_pathname_ioncube"]) . "'" :
					"NULL"
				) .
				", " .
				"'" . $engine->escape ($settings ["awc_dir_install"] . "users/%i/") . "', " .
				"'" .
					$engine->escape (
						"http://" .
						$settings ["apache_hostname"] .
						"/users/%i/htdocs/"
					) .
				"', " .
				"'htdocs/', " .
				"'system/', " .
				"'f', " .
				"'f', " .
				"NULL, " .
				"NULL, " .
				"NULL, " .
				"NULL, " .
				"NULL, " .
				"NULL, " .
				"'t', " .
				"'" . $engine->escape ($connect_info ["mechanism"]) . "', " .
				"'" . $engine->escape ($connect_info ["connect_1"]) . "', " .
				"'" . $engine->escape ($connect_info ["connect_2"]) . "', " .
				"'" . $engine->escape ($settings ["db_admin_username"]) . "', " .
				"'" . $engine->escape ($settings ["db_admin_password"]) . "', " .
				"'f', " .
				"NULL" .
			")"
		) === false) {
			printf ("ERROR: Failed to update the system AWC settings.\n");
			return (false);
		}
		if (self::_int_query (
			$engine,
			"UPDATE admins " .
			"SET " .
				"username='" . $engine->escape ($settings ["awc_admin_username"]) . "', " .
				"password='" . $engine->escape ($settings ["awc_admin_password"]) . "' " .
			"WHERE admin_id='1'"
		) === false) {
			printf ("ERROR: Failed to update the AWC admin data.\n");
			return (false);
		}
		return (true);
	}

	static private function _int_get_connect_info ($settings)
	{
		return (array (
			"engine" => sql_engine::POSTGRESQL,
			"mechanism" =>
				($settings ["db_host"] == "") ?
				sql_mechanism::UNIX :
				sql_mechanism::TCP,
			"connect_1" =>
				($settings ["db_host"] != "") ?
				$settings ["db_host"] :
				null,
			"connect_2" =>
				($settings ["db_port"] != "") ?
				$settings ["db_port"] :
				null
		));
	}

	static private function _int_query ($engine, $query)
	{
		if (($resource = @$engine->query ($query)) == false) {
			return (false);
		}
		$result = array ();
		while (($one_row = $engine->fetch ()) !== false) {
			array_push ($result, $one_row);
		}
		return ($result);
	}

	static public function _int_ask_yes_no ($prompt, $default)
	{
		for (;;) {
			printf ("%s [%s]: ", $prompt, $default ? "Y/n" : "y/N");
			if (($value = stream_get_line (STDIN, 1024, "\n")) !== false) {
				switch ($value) {
				case "y":
					return (true);
				case "n":
					return (false);
				case "":
					return ($default);
				}
				printf ("Please enter y or n\n");
			}
		}
	}

	static private function _int_restart_httpd ()
	{
		exec ("killall -HUP httpd", $output, $result);
		return ($result == 0);
	}
}
