<?php
class install_settings
{
	const FIELD_STRING = 1;
	const FIELD_DIR = 2;

	static public function get ($installer_dir)
	{
		$settings = self::_int_create ();
		self::_int_ask ($settings);
		return (self::_int_fetch ($installer_dir, $settings));
	}

	static private function _int_create ()
	{
		return (array (
			"awc_dir_install" => array (
				"type" => self::FIELD_DIR,
				"description" => "Installation directory",
				"value" => "/usr/local/awc/",
				"allow_empty" => false
			),
			"awc_dir_temp" => array (
				"type" => self::FIELD_DIR,
				"description" => "Temporary files directory (must already exist)",
				"value" => "/tmp/",
				"allow_empty" => false
			),
			"awc_pathname_zend" => array (
				"type" => self::FIELD_STRING,
				"description" => "Pathname of the Zend Encoder (optional)",
				"value" => "",
				"allow_empty" => true
			),
			"awc_pathname_ioncube" => array (
				"type" => self::FIELD_STRING,
				"description" => "Pathname of the IonCube Encoder (optional)",
				"value" => "",
				"allow_empty" => true
			),
			"awc_admin_username" => array (
				"type" => self::FIELD_STRING,
				"description" => "AWC admin username",
				"value" => "admin",
				"allow_empty" => false
			),
			"awc_admin_password" => array (
				"type" => self::FIELD_STRING,
				"description" => "AWC admin password",
				"value" => "admin",
				"allow_empty" => false
			),
			"apache_conf" => array (
				"type" => self::FIELD_STRING,
				"description" => "Location of the httpd.conf file",
				"value" => self::_int_find_http_conf (),
				"allow_empty" => false
			),
			"apache_ip" => array (
				"type" => self::FIELD_STRING,
				"description" => "IP address used by the AWC web server",
				"value" => "127.0.0.1",
				"allow_empty" => false
			),
			"apache_hostname" => array (
				"type" => self::FIELD_STRING,
				"description" => "Hostname used by the AWC web server",
				"value" => "localhost",
				"allow_empty" => false
			),
			"php_ini" => array (
				"type" => self::FIELD_STRING,
				"description" => "Location of the php.ini file",
				"value" => self::_int_find_php_ini (),
				"allow_empty" => false
			),
			"db_host" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL host (leave empty for a local connection)",
				"value" => "",
				"allow_empty" => true
			),
			"db_port" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL port",
				"value" => "",
				"allow_empty" => true
			),
			"db_admin_username" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL database superuser (must already exist)",
				"value" => "postgres",
				"allow_empty" => true
			),
			"db_admin_password" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL database superuser password",
				"value" => "",
				"allow_empty" => true
			),
			"db_web_username" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL database username (will be created)",
				"value" => "awc",
				"allow_empty" => false
			),
			"db_web_password" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL database password",
				"value" => lib_misc::generate_hash (32),
				"allow_empty" => true
			),
			"db_database" => array (
				"type" => self::FIELD_STRING,
				"description" => "PostgreSQL database name (will be created)",
				"value" => "awc",
				"allow_empty" => false
			)
		));
	}

	static private function _int_find_http_conf ()
	{
		return (self::_int_find_generic_file (array (
			"/etc/httpd/conf/httpd.conf",
			"/usr/local/apache2/conf/httpd.conf"
		)));
	}

	static private function _int_find_php_ini ()
	{
		return (self::_int_find_generic_file (array (
			"/etc/php.ini",
			"/usr/local/lib/php.ini"
		)));
	}

	static private function _int_find_generic_file ($locations)
	{
		foreach ($locations as $one_location) {
			if (file_exists ($one_location)) {
				return ($one_location);
			}
		}
		return ("");
	}

	static private function _int_ask (&$settings)
	{
		printf ("\nEnter an empty string to use the default value displayed in []\n\n");
		foreach ($settings as &$one_setting) {
			self::_int_ask_one_item ($one_setting);
		}
	}

	static private function _int_ask_one_item (&$item)
	{
		do {
			printf ("%s [%s]: ", $item ["description"], $item ["value"]);
			if (($value = stream_get_line (STDIN, 1024, "\n")) === false) {
				continue;
			}
			if ($value == "") {
				$value = $item ["value"];
			}
			switch ($item ["type"]) {
			case self::FIELD_STRING:
				break;
			case self::FIELD_DIR:
				if ($value != "") {
					$value = lib_file::path_normalize ($value) . DIRECTORY_SEPARATOR;
				}
				break;
			default:
				printf ("Unsupported field type \"%s\"\n", $item ["type"]);
			}
			if (($value == "") && ($item ["allow_empty"] == false)) {
				print ("Enter a non-empty value.\n");
				continue;
			}
		} while (false);
		$item ["value"] = $value;
	}

	static private function _int_fetch ($installer_dir, $settings)
	{
		$result = array ("installer_dir" => $installer_dir);
		foreach ($settings as $name => $item) {
			$result [$name] = $item ["value"];
		}
		return ($result);
	}
}
?>
