#!/bin/bash

function show_help ()
{
	cat <<END
Usage:
	install.sh [--help]
	install.sh [--php-binary=pathname] [--php-no-version-check]
END
}

function php_locate ()
{
	which php 2> /dev/null
	if [ "$?" != "0" ]; then
		return 1
	fi
	return 0
}

function php_get_version ()
{
	local PHP_BINARY="$1"

	local PHP_OUTPUT

	PHP_OUTPUT="`$PHP_BINARY --version | tr -s \" \" | grep -Ei '^PHP [^ ]+ \(cli\)'`"
	if [ "$?" != "0" ]; then
		return 1
	fi
	echo "$PHP_OUTPUT" | cut -f2 -d " "
	return 0
}

function php_check_version ()
{
	local FOUND_VER="$1"
	local REQUIRED_VER="$2"

	local FOUND_MAJOR
	local FOUND_MINOR
	local FOUND_PIECE
	local FOUND_PATCH
	local FOUND_BETA
	local REQUIRED_MAJOR
	local REQUIRED_MINOR
	local REQUIRED_PATCH

	FOUND_MAJOR=`echo "$FOUND_VER" | cut -f1 -d "."`
	FOUND_MINOR=`echo "$FOUND_VER" | cut -f2 -d "."`
	FOUND_PIECE=`echo "$FOUND_VER" | cut -f3 -d "."`
	FOUND_PATCH=`echo "$FOUND_PIECE" | grep -Eo '^[0-9]+'`
	if echo "$FOUND_PIECE" | grep -Eq '[^0-9].+$'; then
		FOUND_BETA="1"
	else
		FOUND_BETA="0"
	fi
	REQUIRED_MAJOR=`echo "$REQUIRED_VER" | cut -f1 -d "."`
	REQUIRED_MINOR=`echo "$REQUIRED_VER" | cut -f2 -d "."`
	REQUIRED_PATCH=`echo "$REQUIRED_VER" | cut -f3 -d "."`
	compare_values "$FOUND_MAJOR" "$REQUIRED_MAJOR"
	case "$?" in
		255) return 1;;
		1) return 0;;
	esac
	compare_values "$FOUND_MINOR" "$REQUIRED_MINOR"
	case "$?" in
		255) return 1;;
		1) return 0;;
	esac
	compare_values "$FOUND_PATCH" "$REQUIRED_PATCH"
	case "$?" in
		255) return 1;;
		1) return 0;;
	esac
	if [ "$FOUND_BETA" != "0" ]; then
		return 1
	fi
	return 0
}

function compare_values ()
{
	local VALUE_1="$1"
	local VALUE_2="$2"

	if [ $VALUE_1 -lt $VALUE_2 ]; then
		return 255
	fi
	if [ $VALUE_1 -gt $VALUE_2 ]; then
		return 1
	fi
	return 0
}

function check_license_agreement ()
{
	more `dirname $0`/license.txt
	while true
	do
		read -p "Do you accept the license agreement? [Y/N]: "
		case "$REPLY" in
			y | ye | yes | Y | YE | YES) return 0;;
			n | no | N | NO) return 1;;
		esac
	done
}

PHP_BINARY=""
PHP_VERSION_CHECK="1"
PHP_VERSION_REQUIRE="5.1.0"
while [ $# -gt 0 ]
do
	case "$1" in
		--help)
			show_help
			exit 1
			;;
		--php-binary=*)
			PHP_BINARY=${1#'--php-binary='}
			;;
		--php-no-version-check)
			PHP_VERSION_REQUIRE=""
			;;
		*)
			show_help
			exit 1
			;;
	esac
	shift
done

if [ "$PHP_BINARY" = "" ]; then
	PHP_BINARY="`php_locate`"
	if [ "$?" != "0" ]; then
		echo "Could not find the PHP binary"
		exit 2
	fi
else
	if [ ! -x "$PHP_BINARY" ]; then
		echo "Missing excutable $PHP_BINARY"
		exit 2
	fi
fi
if [ "$PHP_VERSION_REQUIRE" != "" ]; then
	PHP_VERSION_FOUND="`php_get_version $PHP_BINARY`"
	if [ "$?" != "0" ]; then
		echo "Could not get the PHP version"
		exit 2
	fi
	php_check_version "$PHP_VERSION_FOUND" "$PHP_VERSION_REQUIRE"
	if [ "$?" != "0" ]; then
		echo "Required PHP $PHP_VERSION_REQUIRE, but only found PHP $PHP_VERSION_FOUND"
		exit 2
	fi
fi
check_license_agreement
if [ "$?" != "0" ]; then
	echo "You must accept the license agreement before installing this software"
	exit 3
fi
"$PHP_BINARY" -f `dirname $0`/install.php
