/*
Requires:
	functions_dom.js
	functions_event.js
*/

function item_drag (item, callback)
{
	this.dom_item = (typeof (item) == 'object') ? item : document.getElementById (item);
	this.callback = callback;
	this.delta_x = null;
	this.delta_y = null;
	this.mouse_move_handler = null;
	this.mouse_move_cookie = null;
	this.mouse_up_handler = null;
	this.mouse_up_cookie = null;

	this._int_drag_prepare ();
}

item_drag.prototype = {
	_int_drag_prepare: function () {
		var this_obj, dom_item, position;

		this_obj = this;
		dom_item = this.dom_item;
		position = dom_get_position (dom_item);
		dom_item.style.position = 'absolute';
		dom_item.style.left = position.x + 'px';
		dom_item.style.top = position.y + 'px';
		event_handler_add (
			dom_item,
			'mousedown',
			function (event) {
				this_obj._int_drag_start (event);
				event_prevent_default (event);			/* NB: Prevent builtin item dragging on Firefox */
			}
		);
		event_handler_add (
			dom_item,
			'dragstart',
			function (event) {event_prevent_default (event);}	/* NB: Prevent builtin item dragging on IE */
		);
		document.body.appendChild (dom_item);
	},

	_int_drag_start: function (event) {
		var this_obj, position, scroll;

		this_obj = this;
		position = dom_get_position (this.dom_item);
		scroll = dom_get_document_scroll ();
		this.delta_x = position.x - (event.clientX + scroll.x);
		this.delta_y = position.y - (event.clientY + scroll.y);
		this.mouse_move_handler = function (event) {this_obj._int_drag_move (event);};
		this.mouse_up_handler = function (event) {this_obj._int_drag_stop (event);};
		this.mouse_move_cookie = event_handler_add (document, 'mousemove', this.mouse_move_handler);
		this.mouse_up_cookie = event_handler_add (document, 'mouseup', this.mouse_up_handler);
		this._int_callback_notify ('start', event);
	},

	_int_drag_move: function (event) {
		var dom_item, scroll;

		dom_item = this.dom_item;
		scroll = dom_get_document_scroll ();
		dom_item.style.left = (event.clientX + scroll.x + this.delta_x) + 'px';
		dom_item.style.top = (event.clientY + scroll.y + this.delta_y) + 'px';
	},

	_int_drag_stop: function (event) {
		this.delta_x = null;
		this.delta_y = null;
		event_handler_remove (this.mouse_move_cookie);
		event_handler_remove (this.mouse_up_cookie);
		this._int_callback_notify ('stop', event);
	},

	_int_callback_notify: function (action, event) {
		var scroll;

		if (this.callback !== null) {
			scroll = dom_get_document_scroll ();
			this.callback (action, {
				dom_item: this.dom_item,
				x: event.clientX + scroll.x,
				y: event.clientY + scroll.y
			});
		}
	}
};
