/*
Requires:
	functions_event.js
*/

function menu_view (dom_container, list, styles, callback)
{
	var i, parent_id;

	this.tree = [];
	for (i in list) {
		this.tree [list [i]['item_id']] = {
			parent_id: list [i]['parent_id'],
			text_left: list [i]['text_left'],
			text_right: list [i]['text_right'],
			dom_element: null,
			dom_menu: null,
			active_id: null,
			children: []
		};
	}
	this.root_id = null;
	for (i in this.tree) {
		if ((parent_id = this.tree [i]['parent_id']) !== null) {
			this.tree [parent_id]['children'].push (i);
		} else {
			this.root_id = i;
		}
	}
	this.styles = styles;
	this.callback = callback;
	this.timer_id = null;

	this._int_create_root_menu (dom_container);
}

menu_view.prototype = {
	hide: function () {
		var active_id;

		if ((active_id = this.tree [this.root_id]['active_id']) !== null) {
			this._int_deactivate_chain (active_id);
		}
	},

	_int_create_root_menu: function (dom_container) {
		var dom_menu_outer, dom_menu_inner, dom_table, dom_tbody, dom_row, dom_cell, children, i;

		dom_menu_outer = document.createElement ('div');
		dom_menu_outer.style.position = 'relative';
		this.tree [this.root_id]['dom_menu'] = dom_menu_outer;
		dom_menu_inner = document.createElement ('div');
		dom_menu_inner.className = this.styles ['root_menu'];
		dom_table = document.createElement ('table');
		dom_table.cellSpacing = '0';
		dom_tbody = document.createElement ('tbody');
		dom_row = document.createElement ('tr');
		dom_row.style.whiteSpace = 'nowrap';
		children = this.tree [this.root_id]['children'];
		for (i in children) {
			dom_cell = this._int_create_root_element (children [i]);
			dom_row.appendChild (dom_cell);
			this.tree [children [i]]['dom_element'] = dom_cell;
		}
		dom_tbody.appendChild (dom_row);
		dom_table.appendChild (dom_tbody);
		dom_menu_inner.appendChild (dom_table);
		dom_menu_outer.appendChild (dom_menu_inner);
		dom_container.appendChild (dom_menu_outer);
	},

	_int_create_root_element: function (item_id) {
		var dom_cell;

		dom_cell = document.createElement ('td');
		dom_cell.className = this.styles ['root_item_low'];
		dom_cell.style.padding = '0.25em 0.5em';
		dom_cell.innerHTML = this.tree [item_id]['text_left'];
		this._int_set_element_events (dom_cell, item_id);
		return (dom_cell);
	},

	_int_create_sub_menu: function (item_id) {
		var dom_menu, dom_table, dom_tbody, dom_row, children, i;

		dom_menu = document.createElement ('div');
		dom_menu.style.position = 'absolute';
		this.tree [item_id]['dom_menu'] = dom_menu;
		this._int_create_sub_position (item_id);
		dom_table = document.createElement ('table');
		dom_table.className = this.styles ['sub_menu'];
		dom_table.cellSpacing = '0';
		dom_tbody = document.createElement ('tbody');
		children = this.tree [item_id]['children'];
			for (i in children) {
		 	dom_row = this._int_create_sub_element (children [i]);
		 	dom_tbody.appendChild (dom_row);
			this.tree [children [i]]['dom_element'] = dom_row;
		}
		dom_table.appendChild (dom_tbody);
		dom_menu.appendChild (dom_table);
		this.tree [this.tree [item_id]['parent_id']]['dom_menu'].appendChild (
			dom_menu
		);
	},

	_int_create_sub_position: function (item_id) {
		var dom_item, dom_menu_sub, dom_menu_parent;

		dom_item = this.tree [item_id]['dom_element'];
		dom_menu_sub = this.tree [item_id]['dom_menu'];
		dom_menu_parent = this.tree [this.tree [item_id]['parent_id']]['dom_menu'];
		/*
			NB: The following observation is valid for IE6 and Firefox. Each table
			is used as an offsetParent for its rows and cells. The offsetTop and
			offsetLeft properties for the rows and cells measure the distance from
			the table border edge to the corresponding row to cell. This differs
			from the description for offsetLeft/offsetTop in the MSDN article
			"Measuring Element Dimension and Location"
			(http://msdn.microsoft.com/workshop/author/om/measuring.asp)
			which shows that the offsetTop/offsetLeft should be measured from
			the padding edge of the offset parent to the border edge of the child.
		*/
		if (this.tree [item_id]['parent_id'] == this.root_id) {
			/*
				NB:
				Under Firefox the submenus of the root menu are positioned N
				pixels to the left from the positioned where they should be,
				where N is the width in pixels of the root menu left border.
				This misplacement is caused by the fact that the table that
				contains the buttons of the root menu does not have a border.
				Instead the root menu style is applied to the inner DIV wrapping
				the root menu table. Thus the border width is not accounted by
				dom_item.offsetLeft

				There is no such misplacement under IE6, because of an IE6 bug.
				offsetParent correctly points to the outer DIV wrapping the
				root menu table. The submenu is using correctly the outer DIV
				padding edge for vertical positioning. However for horizontal
				positioning it is using the table content(?) edge. This bug
				shows when the outer wrapping DIV is relatively positioned, but
				does not show when it is absolutely positioned.
			*/
			dom_menu_sub.style.left = dom_item.offsetLeft + 'px';
			dom_menu_sub.style.top = dom_menu_parent.offsetHeight + 'px';
		} else {
			dom_menu_sub.style.left = dom_menu_parent.offsetWidth + 'px';
			dom_menu_sub.style.top = dom_item.offsetTop + 'px';
		}
	},

	_int_create_sub_element: function (item_id) {
		var dom_row, dom_cell;

		dom_row = document.createElement ('tr');
		dom_row.className = this.styles ['sub_item_low'];
		dom_row.style.whiteSpace = 'nowrap';
		this._int_set_element_events (dom_row, item_id);
		dom_cell = document.createElement ('td');
		dom_cell.style.padding = '0.25em 0.5em';
		dom_cell.innerHTML = this.tree [item_id]['text_left'];
		dom_row.appendChild (dom_cell);
		dom_cell = document.createElement ('td');
		dom_cell.style.padding = '0.25em 0.5em';
		dom_cell.innerHTML = this.tree [item_id]['text_right'];
		dom_row.appendChild (dom_cell);
		return (dom_row);
	},

	_int_set_element_events: function (dom_element, item_id) {
		var this_obj;

		this_obj = this;
		event_handler_add (dom_element, 'mouseover', function () {
			this_obj._int_set_autohide (0);
			this_obj._int_set_active_item (item_id);
		});
		event_handler_add (dom_element, 'mouseout', function () {
			this_obj._int_set_autohide (500);
		});
		event_handler_add (dom_element, 'click', function () {
			this_obj.callback (item_id);
			this_obj.hide ();
		});
	},

	_int_set_active_item: function (item_id) {
		var active_id;

		if ((active_id = this._int_get_active_sibling (item_id)) === item_id) {
			return;
		}
		if (active_id !== null) {
			this._int_deactivate_chain (active_id);
		}
		this._int_activate_item (item_id);
	},

	_int_get_active_sibling: function (item_id) {
		return (this.tree [this.tree [item_id]['parent_id']]['active_id']);
	},

	_int_activate_item: function (item_id) {
		this.tree [this.tree [item_id]['parent_id']]['active_id'] = item_id;
		this.tree [item_id]['dom_element'].className =
			(this.tree [item_id]['parent_id'] == this.root_id) ?
			this.styles ['root_item_high'] :
			this.styles ['sub_item_high'];
		if (this.tree [item_id]['children'].length) {
			this._int_create_sub_menu (item_id);
		}
	},

	_int_deactivate_chain: function (item_id) {
		var active_id;

		for (active_id = item_id; this.tree [active_id]['active_id'] !== null; active_id = this.tree [active_id]['active_id']);
		for (;;) {
			this._int_deactivate_item (active_id);
			if (active_id == item_id) {
				break;
			}
			active_id = this.tree [active_id]['parent_id'];
		}
	},

	_int_deactivate_item: function (item_id) {
		var active_id;

		this.tree [this.tree [item_id]['parent_id']]['active_id'] = null;
		this.tree [item_id]['dom_element'].className =
			(this.tree [item_id]['parent_id'] == this.root_id) ?
			this.styles ['root_item_low'] :
			this.styles ['sub_item_low'];
		for (active_id = item_id; (active_id !== null) && (this.tree [active_id]['dom_menu'] !== null); active_id = this.tree [active_id]['active_id']) {
			this._int_destroy_menu (active_id);
		}
	},

	_int_destroy_menu: function (item_id) {
		var dom_menu, children, i;

		dom_menu = this.tree [this.tree [item_id]['parent_id']]['dom_menu'];
		dom_menu.removeChild (this.tree [item_id]['dom_menu']);
		this.tree [item_id]['dom_menu'] = null;
		children = this.tree [item_id]['children'];
		for (i in children) {
			this.tree [children [i]]['dom_element'] = null;
		}
	},

	_int_set_autohide: function (timeout) {
		var this_obj;

		if (this.timer_id !== null) {
			clearTimeout (this.timer_id);
			this.timer_id = null;
		}
		if (timeout) {
			this_obj = this;
			this.timer_id = setTimeout (function () {this_obj.hide ();}, timeout);
		}
	}
};
