/*
Requires:
	functions_event.js
*/

function tree_view (table_id, image_info, table_builder, list, is_open)
{
	var i, parent_id, last_sibling_index, last_sibling_id;

	this.table_id = table_id;
	this.image_info = image_info;
	this.dom_tbody = null;
	this.table_builder = table_builder;
	this.tree = [];
	for (i in list) {
		this.tree [list [i]['item_id']] = {
			parent_id: list [i]['parent_id'],
			value: list [i]['value'],
			is_visible: false,
			dom_row: null,
			dom_img: null,
			is_last: null,
			state: null,
			children: []
		};
	}
	this.root_id = null;
	for (i in this.tree) {
		parent_id = this.tree [i]['parent_id'];
		if (parent_id !== null) {
			this.tree [parent_id]['children'].push (i);
		} else {
			this.root_id = i;
		}
	}
	for (i in this.tree) {
		parent_id = this.tree [i]['parent_id'];
		if (parent_id !== null) {
			last_sibling_index = this.tree [parent_id]['children'].length - 1;
			last_sibling_id = this.tree [parent_id]['children'][last_sibling_index];
			this.tree [i]['is_last'] = (i == last_sibling_id);
		} else {
			this.tree [i]['is_last'] = true;
		}
		this.tree [i]['state'] =
			(this.tree [i]['children'].length == 0) ?
			'leaf' :
			(is_open ? 'open' : 'closed');
	}
}

tree_view.prototype = {
	is_visible: function () {
		if (this.root_id == null) {
			return (false);
		}
		if (this.tree [this.root_id]['is_visible'] == false) {
			return (false);
		}
		return (true);
	},

	show: function () {
		if (this.is_visible () == true) {
			return;
		}
		this.dom_tbody = document.getElementById (this.table_id).getElementsByTagName ('tbody').item (0);
		this._int_show_item (this.root_id, null);
	},

	hide: function () {
		if (this.is_visible () == false) {
			return;
		}
		this._int_hide_item (this.root_id);
		this.dom_tbody = null;
	},

	open: function (item_id) {
		if (this.tree [item_id]['state'] != 'closed') {
			return;
		}
		this.tree [item_id]['state'] = 'open';
		this._int_update_image (item_id);
		if (this.tree [item_id]['is_visible'] == true) {
			this._int_show_children (item_id);
		}
	},

	close: function (item_id) {
		if (this.tree [item_id]['state'] != 'open') {
			return;
		}
		this.tree [item_id]['state'] = 'closed';
		this._int_update_image (item_id);
		if (this.tree [item_id]['is_visible'] == true) {
			this._int_hide_children (item_id);
		}
	},

	flip: function (item_id) {
		switch (this.tree [item_id]['state']) {
		case 'open':
			this.close (item_id);
			break;
		case 'closed':
			this.open (item_id);
			break;
		}
	},

	invalidate: function (item_id) {
		var old_row;

		if (this.tree [item_id]['is_visible']) {
			old_row = this.tree [item_id]['dom_row'];
			this.tree [item_id]['dom_row'] = null;
			this.tree [item_id]['dom_img'] = null;
			this._int_cache_item (item_id);
			old_row.parentNode.replaceChild (this.tree [item_id]['dom_row'], old_row);
		} else {
			this.tree [item_id]['dom_row'] = null;
			this.tree [item_id]['dom_img'] = null;
		}
	},

	set_root: function (root_id) {
		if (this.is_visible () == true) {
			this.hide ();
			this._int_clear_cache ();
			this.root_id = root_id;
			this.show ();
		} else {
			this._int_clear_cache ();
			this.root_id = root_id;
		}
	},

	get_root: function () {
		return (this.root_id);
	},

	get_item_path: function (item_id) {
		var path;

		path = [];
		while (item_id !== null) {
			path.push ({item_id: item_id, value: this.tree [item_id]['value']});
			item_id = this.tree [item_id]['parent_id'];
		}
		path.reverse ();
		return (path);
	},

	walk_upwards: function (item_id, callback) {
		while (item_id !== null) {
			if (callback (item_id, this.tree [item_id]['value']) == false) {
				return;
			}
			item_id = this.tree [item_id]['parent_id'];
		}
	},

	walk_subtree: function (item_id, callback) {
		var i, child_id;

		if (callback (item_id, this.tree [item_id]['value']) == false) {
			return;
		}
		for (i in this.tree [item_id]['children']) {
			child_id = this.tree [item_id]['children'][i];
			if (this.walk_subtree (child_id, callback) == false) {
				return;
			}
		}
	},

	serialize: function () {
		var list, i;

		list = [];
		list.push (
			this.root_id !== null ?
			this.root_id :
			''
		);
		for (i in this.tree) {
			switch (this.tree [i]['state']) {
			case 'open':
				list.push (i);
				list.push ('1');
				break;
			case 'closed':
				list.push (i);
				list.push ('0');
				break;
			}
		}
		return (list.join (':'));
	},

	unserialize: function (data) {
		var list, item_id, value;

		list = data.split (':');
		if (list.length) {
			item_id = list.shift ();
			if (item_id !== '') {
				this.set_root (item_id);
			}
			while (list.length) {
				item_id = list.shift ();
				value = list.shift ();
				if (item_id in this.tree) {
					if (value == '1') {
						this.open (item_id);
					} else {
						this.close (item_id);
					}
				}
			}
		}
	},

	_int_show_item: function (item_id, dom_prev) {
		this._int_cache_item (item_id);
		dom_prev = this.dom_tbody.insertBefore (
			this.tree [item_id]['dom_row'],
			(dom_prev != null) ? dom_prev.nextSibling : null
		);
		if (this.tree [item_id]['state'] == 'open') {
			dom_prev = this._int_show_children (item_id);
		}
		this.tree [item_id]['is_visible'] = true;
		return (dom_prev);
	},

	_int_hide_item: function (item_id) {
		if (this.tree [item_id]['state'] == 'open') {
			this._int_hide_children (item_id);
		}
		this.dom_tbody.removeChild (this.tree [item_id]['dom_row']);
		this.tree [item_id]['is_visible'] = false;
	},

	_int_show_children: function (item_id) {
		var dom_prev, i;

		dom_prev = this.tree [item_id]['dom_row'];
		for (i in this.tree [item_id]['children']) {
			dom_prev = this._int_show_item (this.tree [item_id]['children'][i], dom_prev);
		}
		return (dom_prev);
	},

	_int_hide_children: function (item_id) {
		var i;

		for (i in this.tree [item_id]['children']) {
			this._int_hide_item (this.tree [item_id]['children'][i]);
		}
	},

	_int_cache_item: function (item_id) {
		var dom_outer_row, dom_outer_cell, dom_inner_table, dom_inner_tbody, dom_inner_row, dom_inner_cell, dom_inner_img;

		if (this.tree [item_id]['dom_row'] != null) {
			return;
		}
		dom_outer_row = document.createElement ('tr');
		dom_outer_row.style.height = this.image_info ['height'] + 'px';
		dom_outer_cell = dom_outer_row.appendChild (document.createElement ('td'));
		dom_inner_table = dom_outer_cell.appendChild (document.createElement ('table'));
		dom_inner_table.style.width = '100%';
		dom_inner_table.style.border = '0';
		dom_inner_table.cellSpacing = '0';
		dom_inner_table.cellPadding = '0';
		dom_inner_tbody = dom_inner_table.appendChild (document.createElement ('tbody'));
		dom_inner_row = dom_inner_tbody.appendChild (document.createElement ('tr'));
		dom_inner_cell = dom_inner_row.appendChild (document.createElement ('td'));
		dom_inner_cell.style.lineHeight = '0px';
		dom_inner_img = this._int_insert_images (dom_inner_cell, item_id);
		this.table_builder (dom_outer_row, item_id, this.tree [item_id]['value']);
		this.tree [item_id]['dom_row'] = dom_outer_row;
		this.tree [item_id]['dom_img'] = dom_inner_img;
		this._int_update_image (item_id);
	},

	_int_clear_cache: function () {
		var i;

		for (i in this.tree) {
			this.tree [i]['dom_row'] = null;
			this.tree [i]['dom_img'] = null;
		}
	},

	_int_insert_images: function (dom_cell, item_id) {
		var this_obj, images, dom_img_first, dom_img_next, current_id;

		images = [];
		dom_img_first = document.createElement ('img');
		dom_img_first.style.width = this.image_info ['width'] + 'px';
		dom_img_first.style.height = this.image_info ['height'] + 'px';
		this_obj = this;
		event_handler_add (dom_img_first, 'click', function (event) {
			this_obj.flip (item_id);
			event_stop_propagation (event);
		});
		images.unshift (dom_img_first);
		if (item_id !== this.root_id) {
			dom_img_next = document.createElement ('img');
			dom_img_next.style.width = this.image_info ['width'] + 'px';
			dom_img_next.style.height = this.image_info ['height'] + 'px';
			dom_img_next.src =
				this.tree [item_id]['is_last'] ?
				this.image_info ['url_corner'] :
				this.image_info ['url_split_right'];
			images.unshift (dom_img_next);
			for (
				current_id = this.tree [item_id]['parent_id'];
				current_id !== this.root_id;
				current_id = this.tree [current_id]['parent_id']
			) {
				dom_img_next = document.createElement ('img');
				dom_img_next.style.width = this.image_info ['width'] + 'px';
				dom_img_next.style.height = this.image_info ['height'] + 'px';
				dom_img_next.src =
					this.tree [current_id]['is_last'] ?
					this.image_info ['url_space'] :
					this.image_info ['url_vertical'];
				images.unshift (dom_img_next);

			}
		}
		dom_cell.style.width = (images.length * this.image_info ['width']) + 'px';
		while (images.length > 0) {
			dom_cell.appendChild (images.shift ());
		}
		return (dom_img_first);
	},

	_int_update_image: function (item_id) {
		if (this.tree [item_id]['dom_img'] == null) {
			return;
		}
		switch (this.tree [item_id]['state']) {
		case 'leaf':
			this.tree [item_id]['dom_img'].src = this.image_info ['url_leaf'];
			break;
		case 'open':
			this.tree [item_id]['dom_img'].src = this.image_info ['url_inner_open'];
			break;
		case 'closed':
			this.tree [item_id]['dom_img'].src = this.image_info ['url_inner_closed'];
			break;
		}
	}
};
