<?php
class field_check
{
	const ERR_BAD_LENGTH_SHORT = "Value is too short";
	const ERR_BAD_LENGTH_LONG = "Value is too long";
	const ERR_BAD_LIST = "Value is not in the list";
	const ERR_BAD_INTEGER = "Value is not an integer";
	const ERR_BAD_INTEGER_SMALL = "Value is too small";
	const ERR_BAD_INTEGER_BIG = "Value is too big";
	const ERR_BAD_FORMAT = "Value has incorrect format";
	const ERR_BAD_UPLOAD = "File is not uploaded";

	static public function check_integer ($value, $min, $max)
	{
		if (preg_match ('/^(-|\\+)?[0-9]+$/', $value) == 0) {
			return (self::ERR_BAD_INTEGER);
		}
		$integer = (int) $value;
		if (($min !== null) && ($integer < $min)) {
			return (self::ERR_BAD_INTEGER_SMALL);
		}
		if (($max !== null) && ($integer > $max)) {
			return (self::ERR_BAD_INTEGER_BIG);
		}
		return (null);
	}

	static public function check_length ($value, $min, $max)
	{
		$length = strlen ($value);
		if (($min !== null) && ($length < $min)) {
			return (self::ERR_BAD_LENGTH_SHORT);
		}
		if (($max !== null) && ($length > $max)) {
			return (self::ERR_BAD_LENGTH_LONG);
		}
		return (null);
	}

	static public function check_list ($value, $list, $allow_empty)
	{
		if ($list === null) {
			return (self::ERR_BAD_LIST);
		}
		$elements = lib_list::get ($list);
		if (
			(($value != "-1") || ($allow_empty == false)) &&
			(array_key_exists ($value, $elements) == false)
		) {
			return (self::ERR_BAD_LIST);
		}
		return (null);
	}

	static public function check_pcre ($value, $pcre)
	{
		if (preg_match ($pcre, $value) == 0) {
			return (self::ERR_BAD_FORMAT);
		}
		return (null);
	}

	static public function check_upload ($value)
	{
		if (is_uploaded_file ($value ["tmp_name"]) == false) {
			return (self::ERR_BAD_UPLOAD);
		}
		return (null);
	}
}
?>
