<?php
class session_manager
{
	const COOKIE_NAME = "_fwk_session";

	private $access;
	private $session_id;
	private $data_old;
	private $data_new;

	public function __construct ($access_type)
	{
		$this->access_type = $access_type;
		$this->_int_session_empty ();
	}

	public function &access ()
	{
		switch ($this->access_type) {
		case const_mvc::SESSION_NONE:
			break;
		case const_mvc::SESSION_READ:
		case const_mvc::SESSION_READWRITE:
			self::_int_session_cleanup ();
			if ($this->_int_session_read ()) {
				$this->_int_session_touch ();
			} else {
				$this->_int_session_create ();
			}
			break;
		default:
			lib_event::fatal (
				"Unknown session access type \"%s\"",
				$this->access_type
			);
		}
		return $this->data_new;
	}

	public function flush ()
	{
		if ($this->data_old !== $this->data_new) {
			switch ($this->access_type) {
			case const_mvc::SESSION_NONE:
			case const_mvc::SESSION_READ:
				lib_event::fatal ("Session access prohibits writing");
			case const_mvc::SESSION_READWRITE:
				$this->_int_session_write ();
				break;
			default:
				lib_event::fatal (
					"Unknown session access type \"%s\"",
					$this->access_type
				);
			}
		}
		$this->_int_session_empty ();
	}

	private function _int_session_empty ()
	{
		$this->session_id = null;
		$this->data_old = null;
		$this->data_new = null;
	}

	static private function _int_session_cleanup ()
	{
		if (mt_rand (0, 99) == 0) {
			lib_event::log (lib_event::L_DEBUG, "Running a sessions cleanup");
			/*
				FIXME: Must avoid blocking on attempts to delete
				sessions that are locked for share/update. Fortunately
				chances of such block happening are almost zero.
			*/
			lib_db::query_raw (
				"DELETE FROM _fwk_sessions " .
				"WHERE current_timestamp - access_time >= '30 minutes'"
			);
		}
	}

	private function _int_session_read ()
	{
		if (array_key_exists (self::COOKIE_NAME, $_COOKIE) == false) {
			return (false);
		}
		if (($session_id = $_COOKIE [self::COOKIE_NAME]) === "") {
			return (false);
		}
		$caps = lib_db::get_capabilities ();
		switch ($this->access_type) {
		case const_mvc::SESSION_NONE:
			lib_event::fatal ("Unexpected session type NONE");
		case const_mvc::SESSION_READ:
			$lock = $caps ["for_share"];
			break;
		case const_mvc::SESSION_READWRITE:
			$lock = $caps ["for_update"];
			break;
		default:
			lib_event::fatal (
				"Unknown session access type \"%s\"",
				$this->access_type
			);
		}
		$rows = lib_db::query_raw (
			"SELECT serialized " .
			"FROM _fwk_sessions " .
			"WHERE " .
				"session_id = '" . lib_db::escape ($session_id) . "' AND " .
				"current_timestamp - access_time < '30 minutes'" .
			(($lock !== null) ? " " . $lock : "")
		);
		if (count ($rows) == 0) {
			return (false);
		}
		$this->session_id = $session_id;
		$this->data_old = unserialize ($rows [0]["serialized"]);
		$this->data_new = $this->data_old;
		return (true);
	}

	private function _int_session_touch ()
	{
		lib_db::query_raw (
			"UPDATE _fwk_sessions " .
			"SET access_time = 'now' " .
			"WHERE session_id = '" . lib_db::escape ($this->session_id) . "'"
		);
	}

	private function _int_session_create ()
	{
		$this->data_old = array ();
		$this->data_new = $this->data_old;
		do {
			$session_id = lib_misc::generate_hash (32);
			lib_db::query_raw (
				"INSERT INTO _fwk_sessions (" .
					"session_id, access_time, serialized" .
				") VALUES (" .
					"'" . lib_db::escape ($session_id) . "', " .
					"'now', " .
					"'" . lib_db::escape (serialize ($this->data_new)) . "'" .
				")"
			);
		} while (lib_db::affected_rows () == 0);
		$this->session_id = $session_id;
		setcookie (self::COOKIE_NAME, $session_id, 0, "/", $_SERVER ["SERVER_NAME"]);
	}

	private function _int_session_write ()
	{
		lib_db::query_raw (
			"UPDATE _fwk_sessions " .
			"SET " .
				"serialized = '" . lib_db::escape (serialize ($this->data_new)) . "', " .
				"access_time = 'now' " .
			"WHERE session_id = '" . lib_db::escape ($this->session_id) . "'"
		);
	}
}
?>
