<?php
class site_action
{
	static public function process ()
	{
		if (array_key_exists ("action", $_GET) == false) {
			lib_event::fatal ("Missing action argument");
		}
		lib_event::log (
			lib_event::L_DEBUG,
			"Started processing of action ID \"%s\"",
			$_GET ["action"]
		);
		$pathname = SYSTEM_DIR . "actions/" . $_GET ["action"] . ".inc";
		if (file_exists ($pathname) == false) {
			lib_event::fatal ("Unknown action \"%s\"", $_GET ["action"]);
		}
		require ($pathname);
		$session_manager = new session_manager ($session_access);
		$session =& $session_manager->access ();
		$field_manager = new field_manager ($fields);
		$fields = $field_manager->array_get ();
		$active_forward = action_handler ($session, $fields);
		$field_manager->array_set ($fields);
		$session_manager->flush ();
		self::_int_forward ($active_forward, $forwards, $field_manager);
		lib_db::commit ();
		lib_event::log (
			lib_event::L_DEBUG,
			"Ended processing of action ID \"%s\"",
			$_GET ["action"]
		);
	}

	static private function _int_forward ($active_forward, $forwards, $field_manager)
	{
		if (is_array ($active_forward) == false) {
			lib_event::fatal ("Action forward is not an array");
		}
		if (count ($active_forward) == 0) {
			lib_event::fatal ("Action forward is an empty array");
		}
		$fwd_name = array_shift ($active_forward);
		if (array_key_exists ($fwd_name, $forwards)) {
			list ($fwd_type, $fwd_target, $fwd_args) = $forwards [$fwd_name];
		} else {
			lib_event::fatal ("Unknown forward name \"%s\"", $fwd_name);
		}
		switch ($fwd_type) {
		case const_mvc::FORWARD_NONE:
			self::_int_forward_args (array (), $active_forward);
			break;
		case const_mvc::FORWARD_TEMPLATE:
			self::_int_forward_template (
				$fwd_target,
				self::_int_forward_args ($fwd_args, $active_forward),
				$field_manager
			);
			break;
		case const_mvc::FORWARD_REDIR_TEMP:
			self::_int_forward_redirect (
				$fwd_target,
				self::_int_forward_args ($fwd_args, $active_forward),
				false
			);
			break;
		case const_mvc::FORWARD_REDIR_PERM:
			self::_int_forward_redirect (
				$fwd_target,
				self::_int_forward_args ($fwd_args, $active_forward),
				true
			);
			break;
		case const_mvc::FORWARD_DOWNLOAD:
			self::_int_forward_download (self::_int_forward_args (
				array ("filename", "mime_type", "data"),
				$active_forward
			));
			break;
		default:
			lib_event::fatal ("Unknown forward type \"%s\"", $fwd_type);
		}
	}

	static private function _int_forward_args ($names, $values)
	{
		$num_names = count ($names);
		$num_values = count ($values);
		if ($num_names != $num_values) {
			lib_event::fatal (
				"Number of argument names (%u) differs from number of argument values (%u)",
				$num_names,
				$num_values
			);
		}
		return ($num_names ? array_combine ($names, $values) : array ());
	}

	static private function _int_forward_template ($template, $args, $field_manager)
	{
		$processor = new template_processor (
			self::_int_get_base_url_path (),
			$field_manager
		);
		$processor->process ($template, $args);
	}

	static private function _int_forward_redirect ($url_partial, $args, $permanent)
	{
		$url_full =
			"http://" .
			$_SERVER ["SERVER_NAME"] .
			(
				($_SERVER ["SERVER_PORT"] != "80") ?
				":" . $_SERVER ["SERVER_PORT"] :
				""
			) .
			(
				(RELOCATABLE == const_mvc::RELOC_NO) ?
				"" :
				self::_int_get_base_url_path ()
			) .
			$url_partial;
		foreach ($args as $key => $value) {
			$url_full .= "&" . $key . "=" . $value;
		}
		lib_event::log (
			lib_event::L_INFO,
			"Sending a %s redirect to \"%s\"",
			$permanent ? "permanent" : "temporary",
			$url_full
		);
		header ($permanent ? "HTTP/1.1 301 Moved Permanently" : "HTTP/1.1 302 Found");
		header ("Location: " . $url_full);
	}

	static private function _int_forward_download ($info)
	{
		header ("Content-Type: " . $info ["mime_type"]);
		header ("Content-Disposition: attachment; filename=\"" . $info ["filename"] . "\"");
		print ($info ["data"]);
	}

	static private function _int_get_base_url_path ()
	{
		if (preg_match ('/^(.*)index\\.php$/', $_SERVER ["SCRIPT_NAME"], $parts) == 0) {
			lib_event::fatal (
				"Failed to get base URL path from the script name \"%s\"",
				$_SERVER ["SCRIPT_NAME"]
			);
		}
		return ($parts [1]);
	}
}
?>
