<?php
class sql_engine_mysql implements sql_engine
{
	private $connect_info;
	private $connection;
	private $query_result;

	public function __construct ($connect_info)
	{
		$this->connect_info = $connect_info;
		$this->connection = null;
		$this->query_result = null;
	}

	public function __destruct ()
	{
		$this->disconnect ();
	}

	static public function is_supported ()
	{
		return (function_exists ("mysql_connect"));
	}

	static public function get_capabilities ()
	{
		return (array (
			"oids" => false,
			"plsql" => false,
			"schemas" => false,
			"sequences" => false,
			"transactions" => sql_transaction::SAVEPOINTS,
			"for_share" => "LOCK IN SHARE MODE",
			"for_update" => "FOR UPDATE"
		));
	}

	public function connect ()
	{
		if ($this->connection !== null) {
			return (true);
		}
		switch ($this->connect_info ["mechanism"]) {
		case sql_mechanism::UNIX:
			$server =
				"localhost" .
				(
					($this->connect_info ["connect_1"] !== null) ?
					":" . $this->connect_info ["connect_1"] :
					""
				);
			break;
		case sql_mechanism::TCP:
			$server =
				$this->connect_info ["connect_1"] .
				(
					($this->connect_info ["connect_2"] !== null) ?
					":" . $this->connect_info ["connect_2"] :
					""
				);
			break;
		default:
			lib_event::fatal (
				"Unknown connection mechanism \"%s\"",
				$this->connect_info ["mechanism"]
			);
		}
		if (($connection = mysql_connect (
			$server,
			($this->connect_info ["username"] !== null) ? $this->connect_info ["username"] : "",
			($this->connect_info ["password"] !== null) ? $this->connect_info ["password"] : "",
			true
		)) === false) {
			return (false);
		}
		if (mysql_select_db ($this->connect_info ["database"], $connection) == false) {
			return (false);
		}
		$this->connection = $connection;
		return (true);
	}

	public function disconnect ()
	{
		if ($this->connection === null) {
			return (true);
		}
		if (mysql_close ($this->connection) === false) {
			return (false);
		}
		$this->connection = null;
		return (true);
	}

	public function escape ($string)
	{
		$this->connect ();
		return (mysql_real_escape_string ($string, $this->connection));
	}

	public function row_to_db ($row, $fields)
	{
		$result = array ();
		reset ($fields);
		foreach ($row as $name => $value_old) {
			$type = each ($fields);
			if ($value_old === null) {
				$value_new = "NULL";
			} else {
				switch ($type ["value"]) {
				case "BOOL":
				case "BOOLEAN":
					$value_new = "'" . ($value_old ? "1" : "0") . "'";
					break;
				default:
					$value_new = "'" . $this->escape ($value_old) . "'";
					break;
				}
			}
			$result [$name] = $value_new;
		}
		return ($result);
	}

	public function row_from_db ($row, $fields)
	{
		$result = array ();
		reset ($fields);
		foreach ($row as $name => $value_old) {
			$type = each ($fields);
			if ($value_old === null) {
				$value_new = null;
			} else {
				switch ($type ["value"]) {
				case "BOOL":
				case "BOOLEAN":
					$value_new = ($value_old == "1");
					break;
				default:
					$value_new = $value_old;
					break;
				}
			}
			$result [$name] = $value_new;
		}
		return ($result);
	}

	public function query ($request)
	{
		if ($this->connect () == false) {
			return (false);
		}
		if (($this->query_result = mysql_query (
			$request,
			$this->connection
		)) === false) {
			return (false);
		}
		return (true);
	}

	public function error ()
	{
		return (mysql_error ($this->connection));
	}

	public function num_fields ()
	{
		if ($this->query_result === true) {
			return (0);
		}
		return (mysql_num_fields ($this->query_result));
	}

	public function fetch ()
	{
		if ($this->query_result === true) {
			return (false);
		}
		return (mysql_fetch_assoc ($this->query_result));
	}

	public function affected_rows ()
	{
		return (mysql_affected_rows ($this->connection));
	}
}
?>
