<?php
class sql_engine_sqlite implements sql_engine
{
	private $connect_info;
	private $pdo;
	private $statement;

	public function __construct ($connect_info)
	{
		$this->connect_info = $connect_info;
		$this->pdo = null;
		$this->statement = null;
	}

	public function __destruct ()
	{
		$this->disconnect ();
	}

	static public function is_supported ()
	{
		return (in_array ("sqlite", PDO::getAVailableDrivers ()));
	}

	static public function get_capabilities ()
	{
		return (array (
			"oids" => false,
			"plsql" => false,
			"schemas" => false,
			"sequences" => false,
			"transactions" => sql_transaction::NONE,
			"for_share" => null,
			"for_update" => null
		));
	}

	public function connect ()
	{
		if ($this->pdo !== null) {
			return (true);
		}
		try {
			$pdo = new PDO ("sqlite:" . $this->connect_info ["connect_1"]);
			$pdo->setAttribute(
				PDO::ATTR_ERRMODE,
				PDO::ERRMODE_WARNING
			);
		} catch (PDOException $exception) {
			return (false);
		}
		$this->pdo = $pdo;
		return (true);
	}

	public function disconnect ()
	{
		$this->pdo = null;
		return (true);
	}

	public function escape ($string)
	{
		return (str_replace ("'", "''", $string));
	}

	public function row_to_db ($row, $fields)
	{
		$result = array ();
		foreach ($row as $name => $value_old) {
			$result [$name] =
				($value_old === null) ?
				"NULL" :
				"'" . pg_escape_string ($value_old) . "'";
		}
		return ($result);
	}

	public function row_from_db ($row, $fields)
	{
		return ($row);
	}

	public function query ($request)
	{
		if ($this->connect () == false) {
			return (false);
		}
		if ($this->statement !== null) {
			if ($this->statement->closeCursor () == false) {
				return (false);
			}
			$this->statement = null;
		}
		if (($this->statement = $this->pdo->query (
			$request
		)) === false) {
			return (false);
		}
		return (true);
	}

	public function error ()
	{
		return (implode (", ", $this->pdo->errorInfo ()));
	}

	public function num_fields ()
	{
		return ($this->statement->columnCount ());
	}

	public function fetch ()
	{
		return ($this->statement->fetch (PDO::FETCH_ASSOC));
	}

	public function affected_rows ()
	{
		return ($this->statement->rowCount ());
	}
}
?>
