<?php
class template_processor
{
	private $base_url_path;
	private $fields;
	private $vars;
	private $src_segments;

	public function __construct ($base_url_path, $fields)
	{
		$this->base_url_path = $base_url_path;
		$this->fields = $fields;
		$this->vars = null;
		$this->src_segments = null;
	}

	public function process ($template, $args)
	{
		lib_event::log (lib_event::L_DEBUG, "Starting processing of template \"%s\"", $template);
		$this->vars = $args;
		print ($this->process_INCLUDE ($template));
		lib_event::log (lib_event::L_DEBUG, "Ending processing of template \"%s\"", $template);
	}

	public function process_INCLUDE ($template)
	{
		lib_event::log (lib_event::L_DEBUG, "Including template \"%s\"", $template);
		$file = SYSTEM_DIR . "templates/" . $template . ".tpl";
		if (($html = require ($file)) === false) {
			lib_event::fatal ("Failed to include the template file \"%s\"", $file);
		}
		return ($html);
	}

	public function process_URL ($relative)
	{
		switch (RELOCATABLE) {
		case const_mvc::RELOC_NO:
			lib_event::fatal ("Unexpected call to process_URL()");
		case const_mvc::RELOC_PARTIALLY:
			return ($this->base_url_path . $relative);
		case const_mvc::RELOC_FULLY:
			$this->_int_set_src_segments ();
			$dest_segments = explode ("/", $relative);
			return (url_relative_path::reference (
				$dest_segments,
				$this->src_segments,
				null
			));
		default:
			lib_event::fatal ("Unknown relocatable type \"%s\"", RELOCATABLE);
		}
	}

	public function process_VAR ($name)
	{
		lib_event::log (lib_event::L_DEBUG, "Outputting variable \"%s\"", $name);
		if (array_key_exists ($name, $this->vars) == false) {
			lib_event::fatal ("Attempt to use an unknown template variable \"%s\"", $name);
		}
		return ($this->vars [$name]);
	}

	public function process_FIELD ($name, $attribs)
	{
		lib_event::log (lib_event::L_DEBUG, "Outputting HTML code for field \"%s\"", $name);
		if ($this->fields->field_exists ($name) == false) {
			lib_event::fatal ("Attempt to get HTML code for an unknown field \"%s\"", $name);
		}
		return ($this->fields->field_build_html ($name, $attribs));
	}

	public function process_ERROR ($name)
	{
		lib_event::log (lib_event::L_DEBUG, "Outputting error code for field \"%s\"", $name);
		if ($this->fields->field_exists ($name) == false) {
			lib_event::fatal ("Attempt to get error text for an unknown field \"%s\"", $name);
		}
		return ($this->fields->field_build_error ($name));
	}

	private function _int_set_src_segments ()
	{
		if ($this->src_segments !== null) {
			return;
		}
		$length = strlen ($this->base_url_path);
		if (substr ($_SERVER ["REQUEST_URI"], 0, $length) != $this->base_url_path) {
			lib_event::fatal (
				"Request for URL \"%s\" which is not inside the base URL path \"%s\"",
				$_SERVER ["REQUEST_URI"],
				$this->base_url_path
			);
		}
		$this->src_segments = explode (
			"/",
			substr ($_SERVER ["REQUEST_URI"], $length)
		);
	}
}
?>
