<?php
class lib_event
{
	const L_DEBUG = 1;
	const L_INFO = 2;
	const L_ERROR = 3;

	const LOG_LIMIT = 4096;

	static public function log ($level, $format)
	{
		if (LOG_ENABLE == false) {
			return;
		}
		/* NB: As of PHP 5.1, func_get_args() cannot be used as a function parameter */
		$args = func_get_args ();
		$message = substr (
			vsprintf ($format, array_slice ($args, 2)),
			0,
			self::LOG_LIMIT
		);
		$string = date ("d-M-Y H:i:s") . " : ";
		switch ($level) {
		case self::L_DEBUG:
			$string .= "DEBUG";
			break;
		case self::L_INFO:
			$string .= "INFO";
			break;
		case self::L_ERROR:
			$string .= "ERROR";
			break;
		default:
			$string .= "UNKNOWN(" . $level . ")";
			break;
		}
		$string .= ": " . $message . "\n";
		error_log ($string, 3, SYSTEM_DIR . "logs/debug.log");
	}

	static public function fatal ($format)
	{
		/* NB: As of PHP 5.1, func_get_args() cannot be used as a function parameter */
		$args = func_get_args ();
		$lines = array_merge (
			array (
				vsprintf ($format, array_slice ($args, 1)),
				"",
				"Backtrace:"
			),
			self::_int_process_backtrace (debug_backtrace ())
		);
		$msg_text = "";
		$msg_html = "";
		foreach ($lines as $one_line) {
			$msg_text .= $one_line . "\n";
			$msg_html .= htmlentities ($one_line) . "<br>";
		}
		self::log (self::L_ERROR, "%s", $msg_text);
		exit ($msg_html);
	}

	static private function _int_process_backtrace ($backtrace)
	{
		$result = array ();
		$i = 0;
		foreach ($backtrace as $item) {
			$line = "#" . $i++ . " ";
			if (array_key_exists ("class", $item)) {
				$line .= $item ["class"] . $item ["type"];
			}
			$line .= $item ["function"];
			if (array_key_exists ("args", $item)) {
				$line .= "(";
				$args_list = array ();
				foreach ($item ["args"] as $arg) {
					array_push ($args_list, print_r ($arg, true));
				}
				$line .= implode (", ", $args_list);
				$line .= ")";
			}
			if (array_key_exists ("file", $item)) {
				$line .= " called at [" . $item ["file"] . ":" . $item ["line"] . "]";
			}
			array_push ($result, $line);
		}
		return ($result);
	}
}
?>
