<?php
class lib_json
{
	static public function build ($value)
	{
		$type = gettype ($value);
		switch ($type) {
		case "array":
			$count = count ($value);
			return (
				(
					($count == 0) ||
					(array_keys ($value) === range (0, $count - 1))
				) ?
				self::_int_build_array ($value) :
				self::_int_build_object ($value)
			);
		case "boolean":
			return ($value ? "true" : "false");
		case "integer":
		case "double":
			return ((string) $value);
		case "NULL":
			return ("null");
		case "string":
			return (self::_int_build_string ($value));
		default:
			lib_event::fatal ("Unsupported element type \"%s\"", $type);
		}
	}

	static private function _int_build_array ($list)
	{
		$elements = array ();
		foreach ($list as $item) {
			array_push ($elements, self::build ($item));
		}
		return ("[" . implode (",", $elements) . "]");
	}

	static private function _int_build_object ($obj)
	{
		$elements = array ();
		foreach ($obj as $key => $value) {
			array_push (
				$elements,
				self::_int_build_string ($key) .
				":" .
				self::build ($value)
			);
		}
		return ("{" . implode (",", $elements) . "}");
	}

	static private function _int_build_string ($value)
	{
		/*
			NB: For performance reasons we do not encode all control characters (required
			by the JSON spec) and do not make sure that the output is UTF-16 encoded (required
			by the ECMA-262 spec).
		*/
		return ("\"" . addcslashes ($value, "\"\\/\x08\x0C\x0A\x0D\x09") . "\"");
	}
}
?>
