<?php
class lib_mail
{
	const PIECE_TEXT = 1;
	const PIECE_NEW_LINE = 2;
	const PIECE_LINK = 3;

	const MULTIPART_DIVIDER = "----=_NextPart_000_0007_01C279BF.1AEA43B0";

	static private $pieces = array ();

	static public function clear ()
	{
		self::$pieces = array ();
	}

	static public function add_text ($text)
	{
		array_push (self::$pieces, array (
			"type" => self::PIECE_TEXT,
			"text" => $text
		));
	}

	static public function add_new_line ()
	{
		array_push (self::$pieces, array ("type" => self::PIECE_NEW_LINE));
	}

	static public function add_link ($url, $text)
	{
		array_push (self::$pieces, array (
			"type" => self::PIECE_LINK,
			"url" => $url,
			"text" => $text
		));
	}

	static public function send ($from, $to, $subject)
	{
		return (mail (
			$to,
			$subject,
			self::_int_get_body (),
			self::_int_get_headers ($from)
		));
	}

	static private function _int_get_body ()
	{
		$result = "This is a multi-part message in MIME format.";
		$result .= self::_int_build_part_text ();
		$result .= self::_int_build_part_html ();
		$result .= self::_int_build_end ();
		return ($result);
	}

	static private function _int_build_part_text ()
	{
		$content = "";
		foreach (self::$pieces as $one_piece) {
			switch ($one_piece ["type"]) {
			case self::PIECE_TEXT:
				$content .= $one_piece ["text"];
				break;
			case self::PIECE_NEW_LINE:
				$content .= "\n";
				break;
			case self::PIECE_LINK:
				$content .= $one_piece ["url"];
				break;
			default:
				lib_event::fatal (
					"Unknown piece type \"%s\"",
					$one_piece ["type"]
				);
			}
		}
		return (self::_int_create_part ("text/plain", $content));
	}

	static private function _int_build_part_html ()
	{
		$content =
			"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n" .
			"<html>\n" .
			"<head>\n" .
			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n" .
			"</head>\n" .
			"<body>\n";
		foreach (self::$pieces as $one_piece) {
			switch ($one_piece ["type"]) {
			case self::PIECE_TEXT:
				$content .= htmlspecialchars ($one_piece ["text"]);
				break;
			case self::PIECE_NEW_LINE:
				$content .= "<br>";
				break;
			case self::PIECE_LINK:
				$content .=
					"<a href=\"" . $one_piece ["url"] . "\">" .
					htmlspecialchars ($one_piece ["text"]) .
					"</a>";
				break;
			default:
				lib_event::fatal (
					"Unknown piece type \"%s\"",
					$one_piece ["type"]
				);
			}
		}
		$content .=
			"</body>\n" .
			"</html>\n";
		return (self::_int_create_part ("text/html", $content));
	}

	static private function _int_create_part ($type, $content)
	{
		return (
			"\n\n" .
			"--" . self::MULTIPART_DIVIDER . "\n" .
			"Content-Type: " . $type . ";\n" .
			"\tcharset=\"iso-8859-1\"\n" .
			"Content-Transfer-Encoding: 8bit\n\n" .
			$content
		);
	}

	static private function _int_build_end ()
	{
		return (
			"\n\n" .
			"--" . self::MULTIPART_DIVIDER . "--\n\n"
		);
	}

	static private function _int_get_headers ($from)
	{
		return (
			"From: " . $from . "\n" .
			"MIME-Version: 1.0\n" .
			"Content-Type: multipart/alternative;\n" .
			"\tboundary=\"" . self::MULTIPART_DIVIDER . "\""
		);
	}
}
?>
