<?php
class lib_nested_set
{
	/*
		$tree elements:
			table,
			filter
			field_id
			field_parent
			field_left
			field_right
	*/
	static public function create ($tree, $data)
	{
		if (($parent_id = $data [$tree ["field_parent"]]) !== null) {
			$nodes = self::_int_insert ($tree, $parent_id);
			$data [$tree ["field_left"]] = $nodes ["left"];
			$data [$tree ["field_right"]] = $nodes ["right"];
		} else {
			$data [$tree ["field_left"]] = "1";
			$data [$tree ["field_right"]] = "28";
		}
		lib_db::create ($tree ["table"], $data);
	}

	static private function _int_insert ($tree, $parent_id)
	{
		$prepared = self::_int_prepare_names ($tree);
		if (($nodes = self::_int_try_insert ($prepared, $parent_id)) === false) {
			self::_int_reindex ($prepared);
			$nodes = self::_int_try_insert ($prepared, $parent_id);
		}
		return ($nodes);
	}

	static private function _int_try_insert ($prepared, $parent_id)
	{
		$filter =
			($prepared ["filter"] != "") ?
			"(" . $prepared ["filter"] . ") AND " :
			"";
		$rows = lib_db::query_raw (
			"SELECT " .
				"coalesce (" .
					"(" .
						"SELECT " . $prepared ["field_right"] . " " .
						"FROM " . $prepared ["table"] . " " .
						"WHERE " .
							$filter .
							$prepared ["field_left"] . " > (" .
								"SELECT " . $prepared ["field_left"] . " " .
								"FROM " . $prepared ["table"] . " " .
								"WHERE " .
									$filter .
									$prepared ["field_id"] . "='" . $parent_id . "'" .
							") AND " .
							$prepared ["field_left"] . " < (" .
								"SELECT " . $prepared ["field_right"] . " " .
								"FROM " . $prepared ["table"] . " " .
								"WHERE " .
									$filter .
									$prepared ["field_id"] . "='" . $parent_id . "'" .
							") " .
						"ORDER BY " . $prepared ["field_right"] . " DESC " .
						"LIMIT 1" .
					"), " .
					"(" .
						"SELECT " . $prepared ["field_left"] . " " .
						"FROM " . $prepared ["table"] . " " .
						"WHERE " .
							$filter .
							$prepared ["field_id"] . "='" . $parent_id . "'" .
					")" .
				") AS left_limit, " .
				"(" .
					"SELECT " . $prepared ["field_right"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $parent_id . "'" .
				") AS right_limit"
		);
		$limit_left = (integer) $rows [0]["left_limit"];
		$limit_right = (integer) $rows [0]["right_limit"];
		$diff = (integer) (($limit_right - $limit_left) / 3);
		return (
			($diff >= 1) ?
			array (
				"left" => $limit_left + $diff,
				"right" => $limit_left + 2 * $diff
			) :
			false
		);
	}

	static private function _int_prepare_names ($tree)
	{
		return (array (
			"table" => lib_db::escape ($tree ["table"]),
			"filter" => lib_db::build_filter ($tree ["table"], $tree ["filter"]),
			"field_id" => lib_db::escape ($tree ["field_id"]),
			"field_parent" => lib_db::escape ($tree ["field_parent"]),
			"field_left" => lib_db::escape ($tree ["field_left"]),
			"field_right" => lib_db::escape ($tree ["field_right"])
		));
	}

	static private function _int_reindex ($prepared)
	{
		if (($root_id = self::_int_find_root ($prepared)) === false) {
			return (false);
		}
		self::_int_reindex_and_invert ($prepared, $root_id, 1);
		self::_int_invert ($prepared);
	}

	static private function _int_find_root ($prepared)
	{
		$rows = lib_db::query_raw (
			"SELECT " . $prepared ["field_id"] . " AS id_field " .
			"FROM " . $prepared ["table"] . " " .
			(
				($prepared ["filter"] != "") ?
				"WHERE " . $prepared ["filter"] . " " :
				""
			) .
			"ORDER BY " . $prepared ["field_left"] . " ASC " .
			"LIMIT 1"
		);
		return (count ($rows) ? $rows [0]["id_field"] : false);
	}

	static private function _int_reindex_and_invert ($prepared, $item_id, $free)
	{
		$next_num = $free;
		$left = -$next_num;
		$next_num += 27;
		while (($child_id = self::_int_find_leftmost_child ($prepared, $item_id)) !== false) {
			$next_num = self::_int_reindex_and_invert ($prepared, $child_id, $next_num);
		}
		$right = -$next_num;
		$next_num += 27;
		lib_db::query_raw (
			"UPDATE " . $prepared ["table"] . " " .
			"SET " .
				$prepared ["field_left"] . " = '" . $left . "', " .
				$prepared ["field_right"] . " = '" . $right . "' " .
			"WHERE " .
				(
					($prepared ["filter"] != "") ?
					"(" . $prepared ["filter"] . ") AND " :
					""
				) .
				$prepared ["field_id"] . "='" . $item_id . "'"
		);
		return ($next_num);
	}

	static private function _int_find_leftmost_child ($prepared, $item_id)
	{
		$filter =
			($prepared ["filter"] != "") ?
			"(" . $prepared ["filter"] . ") AND " :
			"";
		$rows = lib_db::query_raw (
			"SELECT " . $prepared ["field_id"] . " AS id_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				$filter .
				$prepared ["field_left"] . " > (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") AND " .
				$prepared ["field_left"] . " < (" .
					"SELECT " . $prepared ["field_right"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") " .
			"ORDER BY " . $prepared ["field_left"] . " ASC " .
			"LIMIT 1"
		);
		return (count ($rows) ? $rows [0]["id_field"] : false);
	}

	static private function _int_invert ($prepared)
	{
		lib_db::query_raw (
			"UPDATE " . $prepared ["table"] . " " .
			"SET " .
				$prepared ["field_left"] . "=-" . $prepared ["field_left"] . ", " .
				$prepared ["field_right"] . "=-" . $prepared ["field_right"] .
			(
				($prepared ["filter"] != "") ?
				" WHERE " . $prepared ["filter"] :
				""
			)
		);
	}

	static public function move_left ($tree, $item_id)
	{
		$prepared = self::_int_prepare_names ($tree);
		$escaped_id = lib_db::escape ($item_id);
		if (($left_sibling_id = self::_int_find_left_sibling (
			$prepared,
			$escaped_id
		)) === false) {
			return (false);
		}
		self::_int_swap_siblings ($prepared, $left_sibling_id, $escaped_id);
		return (true);
	}

	static private function _int_find_left_sibling ($prepared, $item_id)
	{
		if (($parent_id = self::_int_find_parent ($prepared, $item_id)) === false) {
			return (false);
		}
		$filter =
			($prepared ["filter"] != "") ?
			"(" . $prepared ["filter"] . ") AND " :
			"";
		lib_db::query_raw (
			"SELECT " . $prepared ["field_id"] . " AS id_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				$filter .
				$prepared ["field_left"] . " > (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $parent_id . "'" .
				") AND " .
				$prepared ["field_left"] . " < (" .
					"SELECT " . $prepared ["field_right"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $parent_id . "'" .
				") AND " .
				$prepared ["field_right"] . " < (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") " .
			"ORDER BY " . $prepared ["field_right"] . " DESC " .
			"LIMIT 1"
		);
		return (count ($rows) ? $rows [0]["id_field"] : false);
	}

	static public function move_right ($tree, $item_id)
	{
		$prepared = self::_int_prepare_names ($tree);
		$escaped_id = lib_db::escape ($item_id);
		if (($right_sibling_id = self::_int_find_right_sibling (
			$prepared,
			$escaped_id
		)) === false) {
			return (false);
		}
		self::_int_swap_siblings ($prepared, $escaped_id, $right_sibling_id);
		return (true);
	}

	static private function _int_find_right_sibling ($prepared, $item_id)
	{
		if (($parent_id = self::_int_find_parent ($prepared, $item_id)) === false) {
			return (false);
		}
		$filter =
			($prepared ["filter"] != "") ?
			"(" . $prepared ["filter"] . ") AND " :
			"";
		lib_db::query_raw (
			"SELECT " . $prepared ["field_id"] . " AS id_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				$filter .
				$prepared ["field_left"] . " > (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $parent_id . "'" .
				") AND " .
				$prepared ["field_left"] . " < (" .
					"SELECT " . $prepared ["field_right"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $parent_id . "'" .
				") AND " .
				$prepared ["field_left"] . " > (" .
					"SELECT " . $prepared ["field_right"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") " .
			"ORDER BY " . $prepared ["field_left"] . " ASC " .
			"LIMIT 1"
		);
		return (count ($rows) ? $rows [0]["id_field"] : false);
	}

	static private function _int_find_parent ($prepared, $item_id)
	{
		$filter =
			($prepared ["filter"] != "") ?
			"(" . $prepared ["filter"] . ") AND " :
			"";
		lib_db::query_raw (
			"SELECT " . $prepared ["field_id"] . " AS id_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				$filter .
				$prepared ["field_left"] . " < (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") AND " .
				$prepared ["field_right"] . " > (" .
					"SELECT " . $prepared ["field_left"] . " " .
					"FROM " . $prepared ["table"] . " " .
					"WHERE " .
						$filter .
						$prepared ["field_id"] . "='" . $item_id . "'" .
				") " .
			"ORDER BY " . $prepared ["field_left"] . " DESC " .
			"LIMIT 1"
		);
		return (count ($rows) ? $rows [0]["id_field"] : false);
	}

	static private function _int_swap_siblings ($prepared, $left_sibling_id, $right_sibling_id)
	{
		$rows = lib_db::query_raw (
			"SELECT " .
				$prepared ["field_left"] . " AS left_field, " .
				$prepared ["field_right"] . " AS right_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				(
					($prepared ["filter"] != "") ?
					"(" . $prepared ["filter"] . ") AND " :
					""
				) .
				$prepared ["field_id"] . "='" . $left_sibling_id . "'"
		);
		$left_start = $rows [0]["left_field"];
		$left_end = $rows [0]["right_field"];
		$rows = lib_db::query_raw (
			"SELECT " .
				$prepared ["field_left"] . " AS left_field, " .
				$prepared ["field_right"] . " AS right_field " .
			"FROM " . $prepared ["table"] . " " .
			"WHERE " .
				(
					($prepared ["filter"] != "") ?
					"(" . $prepared ["filter"] . ") AND " :
					""
				) .
				$prepared ["field_id"] . "='" . $right_sibling_id . "'"
		);
		$right_start = $rows [0]["left_field"];
		$right_end = $rows [0]["right_field"];
		$tmp_start = -($left_end - $left_start + 1);
		self::_int_ns_move_range ($prepared, $left_start, $left_end, $tmp_start);
		self::_int_ns_move_range ($prepared, $right_start, $right_end, $left_start);
		self::_int_ns_move_range ($prepared, $tmp_start, -1, $left_start + ($right_end - $right_start) + ($right_start - $left_end));
		RETURN;
	}

	static private function _int_ns_move_range ($prepared, $range_left, $range_right, $new_left)
	{
		$offset = $new_left - $range_left;
		lib_db::query (
			"UPDATE " . $prepared ["table"] . " " .
			"SET " .
				$prepared ["field_left"] . " = " . $prepared ["field_left"] . "+" . $offset . ", " .
				$prepared ["field_right"] . ' = ' . $prepared ["field_right"] . '+' . $offset . ' ' .
			"WHERE " .
				(
					($prepared ["filter"] != "") ?
					"(" . $prepared ["filter"] . ") AND " :
					""
				) .
				$prepared ["field_left"] . " >= " . $range_left . " AND " .
				$prepared ["field_right"] . " <= " . $range_right
		);
	}
}
?>
