/*
Requires:
	functions_dom.js
	functions_event.js
*/

function input_complete (parent_id, callback, values, styles)
{
	this.parent_id = parent_id;
	this.callback = callback;
	this.values = values;
	this.styles= styles;

	this.dom_field = document.getElementById (this.parent_id);
	this.input_table = null;
	this._int_intercept_event ();
}

input_complete.prototype = {
	_int_intercept_event: function () {
		var this_obj;

		this_obj = this;
		event_handler_add (
			this.dom_field,
			('event' in window) ? 'keydown' : 'keypress',
			function (event) {this_obj._event_field_keypress (event);}
		);
		event_handler_add (
			this.dom_field,
			'keyup',
			function (event) {this_obj._event_field_keyup (event);}
		);
		event_handler_add (
			this.dom_field,
			'blur',
			function () {this_obj._event_field_blur ();}
		);
	},

	_event_field_keypress: function (event) {
		if (this.input_table !== null) {
			switch (event.keyCode) {
			case 13:					// Enter
				this._int_select_item ();
				event_prevent_default (event);
				break;
			case 38:					// Up
				this.input_table.set_active_prev ();
				event_prevent_default (event);
				break;
			case 40:					// Down
				this.input_table.set_active_next ();
				event_prevent_default (event);
				break;
			}
		} else {
			if (event.keyCode == 13) {			// Enter
				this.callback ('enter', null);
				event_prevent_default (event);
			}
		}
	},

	_event_field_keyup: function (event) {
		var matches;

		if (
			(event.keyCode != 13) &&
			(event.keyCode != 38) &&
			(event.keyCode != 40)
		) {
			matches = this._int_get_matches ();
			if (matches.length) {
				this._int_table_show ();
				this.input_table.set_matches (matches);
			} else {
				this._int_table_hide ();
			}
		}
	},

	_event_field_blur: function () {
		if (
			(this.input_table !== null) &&
			(this.input_table.is_hovering () == false)
		) {
			this._int_table_hide ();
		}
	},

	_int_get_matches: function () {
		var input, i, matches;

		if ((input = this.dom_field.value.toLowerCase ()) == '') {
			return ([]);
		}
		matches = [];
		for (i in this.values) {
			if (this.values [i]['search'].toLowerCase ().substr (0, input.length) == input) {
				matches.push (i);
			}
		}
		return (matches);
	},

	_int_select_item: function () {
		this.callback ('select', this.input_table.get_active_cookie ());
		this._int_table_hide ();
	},

	_int_table_show: function () {
		var this_obj;

		if (this.input_table === null) {
			this_obj = this;
			this.input_table = new input_table (
				this.styles,
				this.dom_field,
				this.values,
				function () {this_obj._int_select_item ();}
			);
		}
	},

	_int_table_hide: function () {
		if (this.input_table !== null) {
			this.input_table.hide ();
			this.input_table = null;
		}
	}
};

function input_table (styles, dom_field, values, callback)
{
	this.styles = styles;
	this.values = values;
	this.callback = callback;
	this.dom_table = null;
	this.dom_tbody = null;
	this.items = null;
	this.active = null;
	this.hovering = false;

	this._int_create_table (dom_field);
	document.body.appendChild (this.dom_table);
}

input_table.prototype = {
	hide: function () {
		document.body.removeChild (this.dom_table);
		this.dom_table = null;
		this.dom_tbody = null;
		this.items = null;
		this.active = null;
		this.hovering = false;
	},

	set_matches: function (matches) {
		var dom_tbody, items, dom_row, i;

		dom_tbody = this.dom_tbody;
		while (dom_tbody.childNodes.length > 0) {
			dom_tbody.removeChild (dom_tbody.childNodes [0]);
		}
		items = [];
		for (i in matches) {
			dom_row = this._int_create_row (
				i,
				this.values [matches [i]]['text']
			);
			dom_tbody.appendChild (dom_row);
			items [i] = {cookie: matches [i], dom_row: dom_row};
		}
		this.items = items;
		this._int_set_active (0);
	},

	set_active_prev: function () {
		if ((this.active !== null) && (this.active > 0)) {
			this._int_set_active (this.active - 1);
		}
	},

	set_active_next: function () {
		if (
			(this.active !== null) &&
			(this.active < this.dom_table.rows.length - 1)
		) {
			this._int_set_active (this.active + 1);
		}
	},

	get_active_cookie: function () {
		return (
			(this.active !== null) ?
			this.items [this.active]['cookie'] :
			null
		);
	},

	is_hovering: function () {
		return (this.hovering);
	},

	_int_create_table: function (dom_field) {
		var field_pos, dom_table, dom_tbody;

		field_pos = dom_get_position (dom_field);
		dom_table = document.createElement ('table');
		dom_table.className = this.styles ['box'];
		dom_table.style.position = 'absolute';
		dom_table.style.left = field_pos.x + 'px';
		dom_table.style.top = dom_field.offsetHeight + field_pos.y + 2 + 'px';
		dom_table.style.width = dom_field.offsetWidth + 'px';
		dom_tbody = document.createElement ('tbody');
		dom_table.appendChild (dom_tbody);
		this.dom_table = dom_table;
		this.dom_tbody = dom_tbody;
	},

	_int_create_row: function (index, text) {
		var this_obj, dom_row, dom_cell;

		this_obj = this;
		dom_row = document.createElement ('tr');
		dom_row.className = this.styles ['item_low'];
		event_handler_add (
			dom_row,
			'mouseover',
			function (event) {this_obj._event_item_mouseover (index);}
		);
		event_handler_add (
			dom_row,
			'mouseout',
			function (event) {this_obj._event_item_mouseout (index);}
		);
		event_handler_add (
			dom_row,
			'click',
			function (event) {this_obj._event_item_click (index);}
		);
		dom_cell = document.createElement ('td');
		dom_cell.style.whiteSpace = 'nowrap';
		dom_cell.appendChild (document.createTextNode (text));
		dom_row.appendChild (dom_cell);
		return (dom_row);
	},

	_int_set_active: function (index) {
		if (this.active !== null) {
			this.items [this.active]['dom_row'].className = this.styles ['item_low'];
		}
		if (index !== null) {
			this.items [index]['dom_row'].className = this.styles ['item_high'];
		}
		this.active = parseInt (index);
	},

	_event_item_mouseover: function (index) {
		this._int_set_active (index);
		this.hovering = true;
	},

	_event_item_mouseout: function (index) {
		this.hovering = false;
	},

	_event_item_click: function (index) {
		this._int_set_active (index);
		this.callback ();
	}
};
