function message_dispatcher ()
{
	this.handlers = [];
}

message_dispatcher.prototype = {
	add_handler: function (handler, dom_window) {
		var cookie;

		cookie = this.handlers.length;
		this.handlers.push ({
			handler: handler,
			dom_window: dom_window
		});
		dom_window.document ['message_handler_cookie'] = cookie;
		return (cookie);
	},

	process_message: function (code, source_cookie, dest_cookie, info) {
		var result, i;

		result = null;
		for (i in this.handlers) {
			if (this._int_is_valid (i)) {
				if ((result === null) && ((dest_cookie === null) || (i == dest_cookie))) {
					result = this.handlers [i]['handler'].handle (
						code,
						source_cookie,
						info
					);
				}
			} else {
				/* Perform GC of invalid message handlers */
				delete this.handlers [i];
			}
		}
		return (result);
	},

	_int_is_valid: function (cookie) {
		var info, ex;

		/*
		   Verify that the handler is still valid. We require that the handler window is
		   still open and the document in the handler window has not changed. In case
		   document has changed then the handler is invalid  - even if GC has kept our
		   handler intact because of our reference, there is probably some other page
		   displayed in that window not needing any message handling or using other
		   message handler. Also IE6 has a broken GC which releases function code even if
		   we keep a reference to it, so using an obsolete handler would cause a
		   javascript error "Can't execute code from a freed script".

		   For some unknown reason in IE accessing window.closed after calling
		   window.reload(false) for that window, causes a "Permission denied" error, so
		   we have to guard access to the window object with a try/catch block.
		*/
		info = this.handlers [cookie];
		try {
			if (info ['dom_window']['closed']) {
				return (false);
			}
			if (('message_handler_cookie' in info ['dom_window']['document']) == false) {
				return (false);
			}
			if (info ['dom_window']['document']['message_handler_cookie'] != cookie) {
				return (false);
			}
		} catch (ex) {
			return (false);
		}
		return (true);
	}
};
