function time_parse_seconds (epoch_seconds)
{
	return (time_parse_date (
		new Date (epoch_seconds * 1000),
		false
	));
}

/*
   NB: Javascript Date object stores the timestamp internally in UTC format,
   so we need a flag specifying whether to apply local timezone correction
   when parsing the timestamp.
*/
function time_parse_date (date, local)
{
	var day_names = [
		'Monday',
		'Tuesday',
		'Wednesday',
		'Thursday',
		'Friday',
		'Saturday',
		'Sunday'
	];
	var month_names = [
		'January',
		'February',
		'March',
		'April',
		'May',
		'June',
		'July',
		'August',
		'September',
		'October',
		'November',
		'December'
	];
	var year, month, day_of_month, day_of_week, hour_24, hour_12, is_am, minute, second, millisecond;

	year = local ? date.getFullYear () : date.getUTCFullYear ();
	month = (local ? date.getMonth () : date.getUTCMonth ()) + 1;
	day_of_month = local ? date.getDate () : date.getUTCDate ();
	day_of_week = local ? date.getDay () : date.getUTCDay ();
	if (day_of_week == 0) {
		day_of_week = 7;
	}
	hour_24 = local ? date.getHours () : date.getUTCHours ();
	if (hour_24 == 0) {
		hour_12 = 12;
		is_am = true;
	} else if (hour_24 < 12) {
		hour_12 = hour_24;
		is_am = true;
	} else if (hour_24 == 12) {
		hour_12 = 12;
		is_am = false;
	} else {
		hour_12 = hour_24 - 12;
		is_am = false;
	}
	minute = local ? date.getMinutes () : date.getUTCMinutes ();
	second = local ? date.getSeconds () : date.getUTCSeconds ();
	millisecond = local ? date.getMilliseconds () : date.getUTCMilliseconds ();
	return ({
		year: year,
		month: month,
		month_name: month_names [month - 1],
		day_of_month: day_of_month,
		day_of_week: day_of_week,
		day_of_week_name: day_names [day_of_week - 1],
		hour_24: hour_24,
		hour_12: hour_12,
		is_am: is_am,
		minute: minute,
		second: second,
		millisecond: millisecond
	});
}

function strftime (format, seconds)
{
	var time_info, result, i, c;

	time_info = time_parse_seconds (seconds);
	result = '';
	i = 0;
	while (i < format.length) {
		if ((c = format.charAt (i++)) == '%') {
			result += _int_get_time_format (
				format.charAt (i++),
				time_info
			);
		} else {
			result += c;
		}
	}
	return (result);
}

/* FIXME: Must make expansions support fully compatible with PHP strftime() */
function _int_get_time_format (format, info)
{
	switch (format) {
	case '%':
		return ('%');
	case 'a':
		return (info ['day_of_week_name'].substr (0, 3));
	case 'A':
		return (info ['day_of_week_name']);
	case 'b':
		return (info ['month_name'].substr (0, 3));
	case 'B':
		return (info ['month_name']);
	case 'C':
		return (_int_pad_string (Math.floor (info ['year'] / 100), 2, '0'));
	case 'd':
		return (_int_pad_string (info ['day_of_month'], 2, '0'));
	case 'D':
		return (
			_int_pad_string (info ['month'], 2, '0') +
			'/' +
			_int_pad_string (info ['day_of_month'], 2, '0') +
			'/' +
			_int_pad_string (info ['year'] % 100, 2, '0')
		);
	case 'e':
		return (_int_pad_string (info ['day_of_month'], 2, ' '));
	case 'H':
		return (_int_pad_string (info ['hour_24'], 2, '0'));
	case 'I':
		return (_int_pad_string (info ['hour_12'], 2, '0'));
	case 'm':
		return (_int_pad_string (info ['month'], 2, '0'));
	case 'M':
		return (_int_pad_string (info ['minute'], 2, '0'));
	case 'n':
		return ('\n');
	case 'p':
		return (info ['is_am'] ? 'AM' : 'PM');
	case 'r':
		return (
			_int_pad_string (info ['hour_12'], 2, '0') +
			':' +
			_int_pad_string (info ['minute'], 2, '0') +
			':' +
			_int_pad_string (info ['second'], 2, '0') +
			' ' +
			(info ['is_am'] ? 'AM' : 'PM')
		);
	case 'R':
		return (
			_int_pad_string (info ['hour_24'], 2, '0') +
			':' +
			_int_pad_string (info ['minute'], 2, '0')
		);
	case 'S':
		return (_int_pad_string (info ['second'], 2, '0'));
	case 't':
		return ('\t');
	case 'T':
		return (
			_int_pad_string (info ['hour_24'], 2, '0') +
			':' +
			_int_pad_string (info ['minute'], 2, '0') +
			':' +
			_int_pad_string (info ['second'], 2, '0')
		);
	case 'Y':
		return (_int_pad_string (info ['year'], 4, '0'));
	case 'y':
		return (_int_pad_string (info ['year'] % 100, 2, '0'));
	default:
		return (null);
	}
}

function _int_pad_string (value, length, symbol)
{
	value = value.toString ();
	while (value.length < length) {
		value = symbol + value;
	}
	return (value);
}
